@@ -52,6 +52,7 @@ static void clean_exit(UPSCONN_t *ups, char *upsname, char *hostname, int code)
 	free(upsname);
 	free(hostname);
 
+	upscli_sendline(ups, "LOGOUT\n", 7);
 	upscli_disconnect(ups);
 
 	exit(code);
@@ -208,7 +209,6 @@ static int list_upses(const char *name, int verbose)
 		ret = upscli_list_next(&ups, numq, query, &numa, &answer);
 	}
 
-	upscli_disconnect(&ups);
 	return EXIT_SUCCESS;
 }
 
