@@ -1,6 +1,8 @@
 /* upsclient - network communications functions for UPS clients
 
-   Copyright (C) 2002  Russell Kroll <rkroll@exploits.org>
+   Copyright (C)
+	2002	Russell Kroll <rkroll@exploits.org>
+	2008	Arjen de Korte <adkorte-guest@alioth.debian.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +19,7 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
-#include "common.h"
+#include "config.h"	/* safe because it doesn't contain prototypes */
 
 #include <errno.h>
 #include <netdb.h>
@@ -33,6 +35,8 @@
 
 #define UPSCLIENT_MAGIC 0x19980308
 
+#define SMALLBUF	512
+
 #ifdef SHUT_RDWR
 #define shutdown_how SHUT_RDWR
 #else
@@ -89,20 +93,6 @@ struct {
 	{ 0, "Protocol error",			},	/* 42: UPSCLI_ERR_PROTOCOL */
 };
 
-/* make sure we're using a struct that's been through upscli_connect */
-static int upscli_checkmagic(UPSCONN_t *ups)
-{
-	if (!ups) {
-		return 0;
-	}
-
-	if (ups->upsclient_magic != UPSCLIENT_MAGIC) {
-		return 0;
-	}
-
-	return 1;
-}
-
 const char *upscli_strerror(UPSCONN_t *ups)
 {
 #ifdef HAVE_SSL
@@ -114,7 +104,7 @@ const char *upscli_strerror(UPSCONN_t *ups)
 		return upscli_errlist[UPSCLI_ERR_INVALIDARG].str;
 	}
 
-	if (!upscli_checkmagic(ups)) {
+	if (ups->upsclient_magic != UPSCLIENT_MAGIC) {
 		return upscli_errlist[UPSCLI_ERR_INVALIDARG].str;
 	}
 
@@ -155,7 +145,7 @@ const char *upscli_strerror(UPSCONN_t *ups)
 	case 3:		/* parsing (parseconf) error */
 		snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
 			upscli_errlist[ups->upserror].str,
-			ups->pc_ctx->errmsg);
+			ups->pc_ctx.errmsg);
 		return ups->errbuf;
 	}
 
@@ -167,24 +157,28 @@ const char *upscli_strerror(UPSCONN_t *ups)
 	return ups->errbuf;
 }
 
-static void upscli_closefd(UPSCONN_t *ups)
+/* Read up to buflen bytes from fd and return the number of bytes
+   read. If no data is available within d_sec + d_usec, return 0.
+   On error, a value < 0 is returned (errno indicates error). */
+static int select_read(const int fd, void *buf, const size_t buflen, const long d_sec, const long d_usec)
 {
-#ifdef HAVE_SSL
-	if (ups->ssl) {
-		SSL_shutdown(ups->ssl);
-		SSL_free(ups->ssl);
-		ups->ssl = NULL;
+	int		ret;
+	fd_set		fds;
+	struct timeval	tv;
+
+	FD_ZERO(&fds);
+	FD_SET(fd, &fds);
+
+	tv.tv_sec = d_sec;
+	tv.tv_usec = d_usec;
+
+	ret = select(fd + 1, &fds, NULL, NULL, &tv);
+
+	if (ret < 1) {
+		return ret;
 	}
 
-	if (ups->ssl_ctx) {
-		SSL_CTX_free(ups->ssl_ctx);
-		ups->ssl_ctx = NULL;
-	}
-#endif
-
-	shutdown(ups->fd, shutdown_how);
-	close(ups->fd);
-	ups->fd = -1;
+	return read(fd, buf, buflen);
 }
 
 /* internal: abstract the SSL calls for the other functions */
@@ -198,108 +192,90 @@ static int net_read(UPSCONN_t *ups, char *buf, size_t buflen)
 
 		if (ret < 1) {
 			ups->upserror = UPSCLI_ERR_SSLERR;
-			upscli_closefd(ups);
 		}
 
 		return ret;
 	}
 #endif
 
-	ret = read(ups->fd, buf, buflen);
+	ret = select_read(ups->fd, buf, buflen, 5, 0);
 
-	if (ret == 0) {
-		ups->upserror = UPSCLI_ERR_SRVDISC;
-		ups->syserrno = 0;
-		upscli_closefd(ups);
-
-		return -1;
-	}		
-
-	if (ret < 1) {
+	/* error reading data, server disconnected? */
+	if (ret < 0) {
 		ups->upserror = UPSCLI_ERR_READ;
 		ups->syserrno = errno;
+	}
 
-		upscli_closefd(ups);
+	/* no data available, server disconnected? */
+	if (ret == 0) {
+		ups->upserror = UPSCLI_ERR_SRVDISC;
 	}
 
 	return ret;
 }
 
-static int net_write(UPSCONN_t *ups, const char *buf, size_t count)
+/* Write up to buflen bytes to fd and return the number of bytes
+   written. If no data is available within d_sec + d_usec, return 0.
+   On error, a value < 0 is returned (errno indicates error). */
+static int select_write(const int fd, const void *buf, const size_t buflen, const long d_sec, const long d_usec)
+{
+	int		ret;
+	fd_set		fds;
+	struct timeval	tv;
+
+	FD_ZERO(&fds);
+	FD_SET(fd, &fds);
+
+	tv.tv_sec = d_sec;
+	tv.tv_usec = d_usec;
+
+	ret = select(fd + 1, NULL, &fds, NULL, &tv);
+
+	if (ret < 1) {
+		return ret;
+	}
+
+	return write(fd, buf, buflen);
+}
+
+/* internal: abstract the SSL calls for the other functions */
+static int net_write(UPSCONN_t *ups, const char *buf, size_t buflen)
 {
 	int	ret;
 
 #ifdef HAVE_SSL
-
 	if (ups->ssl) {
-		ret = SSL_write(ups->ssl, buf, count);
+		ret = SSL_write(ups->ssl, buf, buflen);
 
 		if (ret < 1) {
 			ups->upserror = UPSCLI_ERR_SSLERR;
-			upscli_closefd(ups);
 		}
 
 		return ret;
 	}
 #endif
 
-	ret = write(ups->fd, buf, count);
+	ret = select_write(ups->fd, buf, buflen, 0, 0);
 
-	if (ret < 1) {
+	/* error writing data, server disconnected? */
+	if (ret < 0) {
 		ups->upserror = UPSCLI_ERR_WRITE;
 		ups->syserrno = errno;
-		upscli_closefd(ups);
+	}
+
+	/* not ready for writing, server disconnected? */
+	if (ret == 0) {
+		ups->upserror = UPSCLI_ERR_SRVDISC;
 	}
 
 	return ret;
 }
 
-/* internal: bring back a line (up to LF or buflen bytes) */
-static int upscli_read(UPSCONN_t *ups, char *buf, size_t buflen)
-{
-	int	ret;
-	size_t	numrec = 0;
-	char	ch;
-
-	if (!ups) {
-		return -1;
-	}
-
-	if ((!buf) || (buflen < 1) || (ups->fd == -1)) {
-		ups->upserror = UPSCLI_ERR_INVALIDARG;
-		return -1;
-	}
-
-	if (!upscli_checkmagic(ups)) {
-		ups->upserror = UPSCLI_ERR_INVALIDARG;
-		return -1;
-	}
-
-	for (;;) {
-		ret = net_read(ups, &ch, 1);
-
-		if (ret < 1) {
-			return -1;	
-		}
-
-		if ((ch == 10) || (numrec == (buflen - 1))) {
-
-			/* tie off the end */
-			buf[numrec] = '\0';
-			return 0;
-		}
-
-		buf[numrec++] = ch;
-	}
-
-	/* NOTREACHED */
-}
-	
 /* stub first */
 #ifndef HAVE_SSL
 static int upscli_sslinit(UPSCONN_t *ups)
 {
-	return 0;		/* not supported */
+	return 0;	/* not supported */
 }
 
 int upscli_sslcert(UPSCONN_t *ups, const char *dir, const char *file, int verify)
@@ -321,47 +297,16 @@ int upscli_sslcert(UPSCONN_t *ups, const char *dir, const char *file, int verify
 
 static int upscli_sslinit(UPSCONN_t *ups)
 {
-	int	ret;
 	char	buf[UPSCLI_NETBUF_LEN];
 
-	if (!ups) {
-		return -1;
-	}
-
-	if (!upscli_checkmagic(ups)) {
-		ups->upserror = UPSCLI_ERR_INVALIDARG;
-		return -1;
-	}
-
 	/* see if upsd even talks SSL/TLS */
 	snprintf(buf, sizeof(buf), "STARTTLS\n");
 
-	ret = write(ups->fd, buf, strlen(buf));
-
-	if (ret < 0) {
-		ups->upserror = UPSCLI_ERR_WRITE;
-		ups->syserrno = errno;
-
-		upscli_closefd(ups);
+	if (upscli_sendline(ups, buf, strlen(buf)) != 0) {
 		return -1;
 	}
 
-	memset(buf, '\0', sizeof(buf));
-	ret = read(ups->fd, buf, sizeof(buf));
-
-	if (ret == 0) {			/* server hung up on us! */
-		ups->upserror = UPSCLI_ERR_SRVDISC;
-		ups->syserrno = 0;
-
-		upscli_closefd(ups);
-		return -1;
-	}
-
-	if (ret < 1) {
-		ups->upserror = UPSCLI_ERR_READ;
-		ups->syserrno = errno;
-
-		upscli_closefd(ups);
+	if (upscli_readline(ups, buf, sizeof(buf)) != 0) {
 		return -1;
 	}
 
@@ -386,9 +331,7 @@ static int upscli_sslinit(UPSCONN_t *ups)
 		return 0;
 	}
 
-	ret = SSL_set_fd(ups->ssl, ups->fd);
-
-	if (ret != 1) {
+	if (SSL_set_fd(ups->ssl, ups->fd) != 1) {
 		return -1;
 	}
 
@@ -449,44 +392,40 @@ int upscli_connect(UPSCONN_t *ups, const char *host, int port, int flags)
 	int			v;
 #endif
 
-	/* clear out any lingering junk */
-	ups->fd = -1;
-	ups->host = NULL;
-	ups->flags = 0;
-	ups->upserror = 0;
-	ups->syserrno = 0;
-	ups->upsclient_magic = UPSCLIENT_MAGIC;
-
-	if ((ups->pc_ctx = malloc(sizeof(PCONF_CTX_t))) == NULL) {
-		ups->upserror = UPSCLI_ERR_NOMEM;
+	if (!ups) {
 		return -1;
 	}
 
-	pconf_init(ups->pc_ctx, NULL);
+	/* clear out any lingering junk */
+	memset(ups, 0, sizeof(*ups));
+	ups->upsclient_magic = UPSCLIENT_MAGIC;
+	ups->fd = -1;
 
-	ups->ssl_ctx = NULL;
-	ups->ssl = NULL;
+	pconf_init(&ups->pc_ctx, NULL);
 
 	if (!host) {
 		ups->upserror = UPSCLI_ERR_NOSUCHHOST;
+		upscli_disconnect(ups);
 		return -1;
 	}
 
 #ifndef	HAVE_IPV6
 	serv = gethostbyname(host);
 
-	if (serv == NULL) {
+	if (!serv) {
 		struct  in_addr	listenaddr;
 
 		if (!inet_aton(host, &listenaddr)) {
 			ups->upserror = UPSCLI_ERR_NOSUCHHOST;
+			upscli_disconnect(ups);
 			return -1;
 		}
 
 		serv = gethostbyaddr(&listenaddr, sizeof(listenaddr), AF_INET);
 
-		if (serv == NULL) {
+		if (!serv) {
 			ups->upserror = UPSCLI_ERR_NOSUCHHOST;
+			upscli_disconnect(ups);
 			return -1;
 		}
 	}
@@ -494,38 +433,37 @@ int upscli_connect(UPSCONN_t *ups, const char *host, int port, int flags)
 	if ((ups->fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 		ups->upserror = UPSCLI_ERR_SOCKFAILURE;
 		ups->syserrno = errno;
+		upscli_disconnect(ups);
 		return -1;
 	}
 
-	memset(&local, '\0', sizeof(struct sockaddr_in));
+	memset(&local, '\0', sizeof(local));
 	local.sin_family = AF_INET;
 	local.sin_port = htons(INADDR_ANY);
 
-	memset(&server, '\0', sizeof(struct sockaddr_in));
+	memset(&server, '\0', sizeof(server));
 	server.sin_family = AF_INET;
 	server.sin_port = htons(port);
 
 	memcpy(&server.sin_addr, serv->h_addr, serv->h_length);
 
-	if (bind(ups->fd, (struct sockaddr *) &local, sizeof(struct sockaddr_in)) == -1) {
+	if (bind(ups->fd, (struct sockaddr *) &local, sizeof(local)) < 0) {
 		ups->upserror = UPSCLI_ERR_BINDFAILURE;
 		ups->syserrno = errno;
-		close(ups->fd);
-		ups->fd = -1;
+		upscli_disconnect(ups);
 		return -1;
 	}
 
-	if (connect(ups->fd, (struct sockaddr *) &server, sizeof(struct sockaddr_in)) == -1) {
+	if (connect(ups->fd, (struct sockaddr *) &server, sizeof(struct sockaddr_in)) < 0) {
 		ups->upserror = UPSCLI_ERR_CONNFAILURE;
 		ups->syserrno = errno;
-		close(ups->fd);
-		ups->fd = -1;
+		upscli_disconnect(ups);
 		return -1;
 	}
 #else
-	snprintf(sport, NI_MAXSERV, "%hu", (unsigned short int)port);
+	snprintf(sport, sizeof(sport), "%hu", (unsigned short int)port);
 
-	memset (&hints, 0, sizeof (struct addrinfo));
+	memset(&hints, 0, sizeof(hints));
 
 	if (flags & UPSCLI_CONN_INET6) {
 		hints.ai_family = AF_INET6;
@@ -539,14 +477,17 @@ int upscli_connect(UPSCONN_t *ups, const char *host, int port, int flags)
 	hints.ai_protocol = IPPROTO_TCP;
 
 	while ((v = getaddrinfo(host, sport, &hints, &res)) != 0) {
-		switch (v) {
+		switch (v)
+		{
 		case EAI_AGAIN:
 			continue;
 		case EAI_NONAME:
 			ups->upserror = UPSCLI_ERR_NOSUCHHOST;
+			upscli_disconnect(ups);
 			return -1;
 		case EAI_MEMORY:
 			ups->upserror = UPSCLI_ERR_NOMEM;
+			upscli_disconnect(ups);
 			return -1;
 		case EAI_SYSTEM:
 			ups->syserrno = errno;
@@ -554,6 +495,7 @@ int upscli_connect(UPSCONN_t *ups, const char *host, int port, int flags)
 		}
 
 		ups->upserror = UPSCLI_ERR_UNKNOWN;
+		upscli_disconnect(ups);
 		return -1;
 	}
 
@@ -600,14 +542,15 @@ int upscli_connect(UPSCONN_t *ups, const char *host, int port, int flags)
 	freeaddrinfo(res);
 
 	if (ups->fd < 0) {
+		upscli_disconnect(ups);
 		return -1;
 	}
 #endif
+	ups->host = strdup(host);
 
-	if ((ups->host = strdup(host)) == NULL) {
+	if (!ups->host) {
 		ups->upserror = UPSCLI_ERR_NOMEM;
-		close(ups->fd);
-		ups->fd = -1;
+		upscli_disconnect(ups);
 		return -1;
 	}
 
@@ -618,18 +561,20 @@ int upscli_connect(UPSCONN_t *ups, const char *host, int port, int flags)
 
 		/* see if something made us die inside sslinit */
 		if (ups->upserror != 0) {
+			upscli_disconnect(ups);
 			return -1;
 		}
 	}
 
 	if ((flags & UPSCLI_CONN_REQSSL) && (upscli_sslinit(ups) != 1)) {
 		ups->upserror = UPSCLI_ERR_SSLFAIL;
-		upscli_closefd(ups);
+		upscli_disconnect(ups);
 		return -1;
 	}
 		
 	return 0;
 }
+
 /* map upsd error strings back to upsclient internal numbers */
 static struct {
 	int	errnum;
@@ -747,9 +692,12 @@ static int verify_resp(int num, const char **q, char **a)
 int upscli_get(UPSCONN_t *ups, unsigned int numq, const char **query, 
 		unsigned int *numa, char ***answer)
 {
-	int	ret;
 	char	cmd[UPSCLI_NETBUF_LEN], tmp[UPSCLI_NETBUF_LEN];
 	
+	if (!ups) {
+		return -1;
+	}
+
 	if (numq < 1) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
 		return -1;
@@ -758,13 +706,11 @@ int upscli_get(UPSCONN_t *ups, unsigned int numq, const char **query,
 	/* create the string to send to upsd */
 	build_cmd(cmd, sizeof(cmd), "GET", numq, query);
 
-	ret = net_write(ups, cmd, strlen(cmd));
-
-	if (ret < 0) {
+	if (upscli_sendline(ups, cmd, strlen(cmd)) != 0) {
 		return -1;
 	}
 
-	if (upscli_read(ups, tmp, sizeof(tmp)) != 0) {
+	if (upscli_readline(ups, tmp, sizeof(tmp)) != 0) {
 		return -1;
 	}
 
@@ -772,7 +718,7 @@ int upscli_get(UPSCONN_t *ups, unsigned int numq, const char **query,
 		return -1;
 	}
 
-	if (!pconf_line(ups->pc_ctx, tmp)) {
+	if (!pconf_line(&ups->pc_ctx, tmp)) {
 		ups->upserror = UPSCLI_ERR_PARSE;
 		return -1;
 	}
@@ -780,27 +726,30 @@ int upscli_get(UPSCONN_t *ups, unsigned int numq, const char **query,
 	/* q: [GET] VAR <ups> <var>   *
 	 * a: VAR <ups> <var> <val> */
 
-	if (ups->pc_ctx->numargs < numq) {
+	if (ups->pc_ctx.numargs < numq) {
 		ups->upserror = UPSCLI_ERR_PROTOCOL;
 		return -1;
 	}
 
-	if (!verify_resp(numq, query, ups->pc_ctx->arglist)) {
+	if (!verify_resp(numq, query, ups->pc_ctx.arglist)) {
 		ups->upserror = UPSCLI_ERR_PROTOCOL;
 		return -1;
 	}
 
-	*numa = ups->pc_ctx->numargs;
-	*answer = ups->pc_ctx->arglist;
+	*numa = ups->pc_ctx.numargs;
+	*answer = ups->pc_ctx.arglist;
 
 	return 0;
 }
 
 int upscli_list_start(UPSCONN_t *ups, unsigned int numq, const char **query)
 {
-	int	ret;
 	char	cmd[UPSCLI_NETBUF_LEN], tmp[UPSCLI_NETBUF_LEN];
 
+	if (!ups) {
+		return -1;
+	}
+
 	if (numq < 1) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
 		return -1;
@@ -809,13 +758,11 @@ int upscli_list_start(UPSCONN_t *ups, unsigned int numq, const char **query)
 	/* create the string to send to upsd */
 	build_cmd(cmd, sizeof(cmd), "LIST", numq, query);
 
-	ret = net_write(ups, cmd, strlen(cmd));
-
-	if (ret < 0) {
+	if (upscli_sendline(ups, cmd, strlen(cmd)) != 0) {
 		return -1;
 	}
 
-	if (upscli_read(ups, tmp, sizeof(tmp)) != 0) {
+	if (upscli_readline(ups, tmp, sizeof(tmp)) != 0) {
 		return -1;
 	}
 
@@ -823,19 +770,19 @@ int upscli_list_start(UPSCONN_t *ups, unsigned int numq, const char **query)
 		return -1;
 	}
 
-	if (!pconf_line(ups->pc_ctx, tmp)) {
+	if (!pconf_line(&ups->pc_ctx, tmp)) {
 		ups->upserror = UPSCLI_ERR_PARSE;
 		return -1;
 	}
 
-	if (ups->pc_ctx->numargs < 2) {
+	if (ups->pc_ctx.numargs < 2) {
 		ups->upserror = UPSCLI_ERR_PROTOCOL;
 		return -1;
 	}
 
 	/* the response must start with BEGIN LIST */
-	if ((strcasecmp(ups->pc_ctx->arglist[0], "BEGIN") != 0) || 
-		(strcasecmp(ups->pc_ctx->arglist[1], "LIST") != 0)) {
+	if ((strcasecmp(ups->pc_ctx.arglist[0], "BEGIN") != 0) ||
+		(strcasecmp(ups->pc_ctx.arglist[1], "LIST") != 0)) {
 		ups->upserror = UPSCLI_ERR_PROTOCOL;
 		return -1;
 	}
@@ -845,7 +792,7 @@ int upscli_list_start(UPSCONN_t *ups, unsigned int numq, const char **query)
 
 	/* compare q[0]... to a[2]... */
 
-	if (!verify_resp(numq, query, &ups->pc_ctx->arglist[2])) {
+	if (!verify_resp(numq, query, &ups->pc_ctx.arglist[2])) {
 		ups->upserror = UPSCLI_ERR_PROTOCOL;
 		return -1;
 	}
@@ -858,7 +805,11 @@ int upscli_list_next(UPSCONN_t *ups, unsigned int numq, const char **query,
 {
 	char	tmp[UPSCLI_NETBUF_LEN];
 
-	if (upscli_read(ups, tmp, sizeof(tmp)) != 0) {
+	if (!ups) {
+		return -1;
+	}
+
+	if (upscli_readline(ups, tmp, sizeof(tmp)) != 0) {
 		return -1;
 	}
 
@@ -866,30 +817,30 @@ int upscli_list_next(UPSCONN_t *ups, unsigned int numq, const char **query,
 		return -1;
 	}
 
-	if (!pconf_line(ups->pc_ctx, tmp)) {
+	if (!pconf_line(&ups->pc_ctx, tmp)) {
 		ups->upserror = UPSCLI_ERR_PARSE;
 		return -1;
 	}
 
-	if (ups->pc_ctx->numargs < 1) {
+	if (ups->pc_ctx.numargs < 1) {
 		ups->upserror = UPSCLI_ERR_PROTOCOL;
 		return -1;
 	}
 
-	*numa = ups->pc_ctx->numargs;
-	*answer = ups->pc_ctx->arglist;
+	*numa = ups->pc_ctx.numargs;
+	*answer = ups->pc_ctx.arglist;
 
 	/* see if this is the end */
-	if (ups->pc_ctx->numargs >= 2) {
-		if ((!strcmp(ups->pc_ctx->arglist[0], "END")) &&
-			(!strcmp(ups->pc_ctx->arglist[1], "LIST")))
+	if (ups->pc_ctx.numargs >= 2) {
+		if ((!strcmp(ups->pc_ctx.arglist[0], "END")) &&
+			(!strcmp(ups->pc_ctx.arglist[1], "LIST")))
 			return 0;
 	}
 
 	/* q: VAR <ups> */
 	/* a: VAR <ups> <val> */
 
-	if (!verify_resp(numq, query, ups->pc_ctx->arglist)) {
+	if (!verify_resp(numq, query, ups->pc_ctx.arglist)) {
 		ups->upserror = UPSCLI_ERR_PROTOCOL;
 		return -1;
 	}
@@ -906,19 +857,25 @@ int upscli_sendline(UPSCONN_t *ups, const char *buf, size_t buflen)
 		return -1;
 	}
 
-	if ((!buf) || (buflen < 1) || (ups->fd == -1)) {
+	if (ups->fd == -1) {
+		ups->upserror = UPSCLI_ERR_DRVNOTCONN;
+		return -1;
+	}
+
+	if ((!buf) || (buflen < 1)) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
 		return -1;
 	}
 
-	if (!upscli_checkmagic(ups)) {
+	if (ups->upsclient_magic != UPSCLIENT_MAGIC) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
 		return -1;
 	}
 
 	ret = net_write(ups, buf, buflen);
 
-	if (ret < 0) {
+	if (ret < 1) {
+		upscli_disconnect(ups);
 		return -1;
 	}
 
@@ -927,53 +884,72 @@ int upscli_sendline(UPSCONN_t *ups, const char *buf, size_t buflen)
 
 int upscli_readline(UPSCONN_t *ups, char *buf, size_t buflen)
 {
-	char	tmp[LARGEBUF];
+	int	ret;
+	size_t	recv;
 
 	if (!ups) {
 		return -1;
 	}
 
-	if ((!buf) || (buflen < 1) || (ups->fd == -1)) {
+	if (ups->fd == -1) {
+		ups->upserror = UPSCLI_ERR_DRVNOTCONN;
+		return -1;
+	}
+
+	if ((!buf) || (buflen < 1)) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
 		return -1;
 	}
 
-	if (!upscli_checkmagic(ups)) {
+	if (ups->upsclient_magic != UPSCLIENT_MAGIC) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
 		return -1;
 	}
 
-	if (upscli_read(ups, tmp, sizeof(tmp)) != 0) {
-		return -1;
+	for (recv = 0; recv < (buflen-1); recv++) {
+
+		if (ups->readidx == ups->readlen) {
+
+			ret = net_read(ups, ups->readbuf, sizeof(ups->readbuf));
+
+			if (ret < 1) {
+				upscli_disconnect(ups);
+				return -1;
+			}
+
+			ups->readlen = ret;
+			ups->readidx = 0;
+		}
+
+		buf[recv] = ups->readbuf[ups->readidx++];
+
+		if (buf[recv] == '\n') {
+			break;
+		}
 	}
 
-	if (upscli_errcheck(ups, tmp) != 0) {
-		return -1;
-	}
-
-	snprintf(buf, buflen, "%s", tmp);
-
+	buf[recv] = '\0';
 	return 0;
 }
 
 /* split upsname[@hostname[:port]] into separate components */
 int upscli_splitname(const char *buf, char **upsname, char **hostname, int *port)
 {
-	char	*s, tmp[SMALLBUF];
+	char	*s, tmp[SMALLBUF], *last = NULL;
 
 	/* paranoia */
 	if ((!buf) || (!upsname) || (!hostname) || (!port)) {
 		return -1;
 	}
 
-	if (snprintf(tmp, SMALLBUF, "%s", buf) < 1) {
+	if (snprintf(tmp, sizeof(tmp), "%s", buf) < 1) {
 		fprintf(stderr, "upscli_splitname: can't parse empty string\n");
 		return -1;
 	}
 
 	s = strchr(tmp, '@');
 
-	if ((*upsname = strdup(strtok(tmp, "@"))) == NULL) {
+	if ((*upsname = strdup(strtok_r(tmp, "@", &last))) == NULL) {
 		fprintf(stderr, "upscli_splitname: strdup failed\n");
 		return -1;
 	}
@@ -995,14 +971,14 @@ int upscli_splitname(const char *buf, char **upsname, char **hostname, int *port
 /* split hostname[:port] into separate components */
 int upscli_splitaddr(const char *buf, char **hostname, int *port)
 {
-	char	*s, tmp[SMALLBUF];
+	char	*s, tmp[SMALLBUF], *last = NULL;
 
 	/* paranoia */
 	if ((!buf) || (!hostname) || (!port)) {
 		return -1;
 	}
 
-	if (snprintf(tmp, SMALLBUF, "%s", buf) < 1) {
+	if (snprintf(tmp, sizeof(tmp), "%s", buf) < 1) {
 		fprintf(stderr, "upscli_splitaddr: can't parse empty string\n");
 		return -1;
 	}
@@ -1013,20 +989,20 @@ int upscli_splitaddr(const char *buf, char **hostname, int *port)
 			return -1;
 		}
 
-		if ((*hostname = strdup(strtok(tmp+1, "]"))) == NULL) {
+		if ((*hostname = strdup(strtok_r(tmp+1, "]", &last))) == NULL) {
 			fprintf(stderr, "upscli_splitaddr: strdup failed\n");
 			return -1;
 		}
 
 		/* no port specified, use default */
-		if (((s = strtok(NULL, "\0")) == NULL) || (*s != ':')) {
+		if (((s = strtok_r(NULL, "\0", &last)) == NULL) || (*s != ':')) {
 			*port = PORT;
 			return 0;
 		}
 	} else {
 		s = strchr(tmp, ':');
 
-		if ((*hostname = strdup(strtok(tmp, ":"))) == NULL) {
+		if ((*hostname = strdup(strtok_r(tmp, ":", &last))) == NULL) {
 			fprintf(stderr, "upscli_splitaddr: strdup failed\n");
 			return -1;
 		}
@@ -1052,28 +1028,36 @@ int upscli_disconnect(UPSCONN_t *ups)
 		return -1;
 	}
 
-	if (!upscli_checkmagic(ups)) {
+	if (ups->upsclient_magic != UPSCLIENT_MAGIC) {
 		return -1;
 	}
 
-	if (ups->fd != -1) {
-
-		/* try to disconnect gracefully */
-		net_write(ups, "LOGOUT\n", 7);
-
-		upscli_closefd(ups);
-	}
-
-	if (ups->pc_ctx) {
-		pconf_finish(ups->pc_ctx);
-		free(ups->pc_ctx);
-		ups->pc_ctx = NULL;
-	}
+	pconf_finish(&ups->pc_ctx);
 
 	free(ups->host);
-
 	ups->host = NULL;
-	ups->upsclient_magic = 0;
+
+	if (ups->fd < 0) {
+		return 0;
+	}
+
+#ifdef HAVE_SSL
+	if (ups->ssl) {
+		SSL_shutdown(ups->ssl);
+		SSL_free(ups->ssl);
+		ups->ssl = NULL;
+	}
+
+	if (ups->ssl_ctx) {
+		SSL_CTX_free(ups->ssl_ctx);
+		ups->ssl_ctx = NULL;
+	}
+#endif
+
+	shutdown(ups->fd, shutdown_how);
+
+	close(ups->fd);
+	ups->fd = -1;
 
 	return 0;
 }
@@ -1084,7 +1068,7 @@ int upscli_fd(UPSCONN_t *ups)
 		return -1;
 	}
 
-	if (!upscli_checkmagic(ups)) {
+	if (ups->upsclient_magic != UPSCLIENT_MAGIC) {
 		return -1;
 	}
 
@@ -1097,7 +1081,7 @@ int upscli_upserror(UPSCONN_t *ups)
 		return -1;
 	}
 
-	if (!upscli_checkmagic(ups)) {
+	if (ups->upsclient_magic != UPSCLIENT_MAGIC) {
 		return -1;
 	}
 
@@ -1110,7 +1094,7 @@ int upscli_ssl(UPSCONN_t *ups)
 		return -1;
 	}
 
-	if (!upscli_checkmagic(ups)) {
+	if (ups->upsclient_magic != UPSCLIENT_MAGIC) {
 		return -1;
 	}
 
