@@ -43,7 +43,7 @@ typedef struct {
 	int	syserrno;
 	int	upsclient_magic;
 
-	PCONF_CTX_t	*pc_ctx;
+	PCONF_CTX_t	pc_ctx;
 
 	char	errbuf[UPSCLI_ERRBUF_LEN];
 
@@ -55,6 +55,10 @@ typedef struct {
 	void	*ssl;
 #endif
 
+	char	readbuf[64];
+	size_t	readlen;
+	size_t	readidx;
+
 }	UPSCONN_t;
 
 /* manage backward compatibility for external client application */
