@@ -787,6 +787,9 @@ static void drop_connection(utype_t *ups)
 	clearflag(&ups->status, ST_LOGIN);
 	clearflag(&ups->status, ST_CONNECTED);
 
+	/* Attempt to logout from this UPS. If this doesn't work,
+	don't bother (it may not be listening to us anymore) */
+	upscli_sendline(&ups->conn, "LOGOUT\n", 7);
 	upscli_disconnect(&ups->conn);
 }
 
@@ -1316,7 +1319,7 @@ static void upsmon_cleanup(void)
 	while (utmp) {
 		unext = utmp->next;
 
-		upscli_disconnect(&utmp->conn);
+		drop_connection(utmp);
 		ups_free(utmp);
 
 		utmp = unext;
