@@ -9,6 +9,6 @@
 /* track client connections */
 struct conn_t {
 	int     fd;
-	PCONF_CTX	ctx;
+	PCONF_CTX_t	ctx;
 	void    *next;
 };
