@@ -369,21 +369,23 @@ void upsdebugx(int level, const char *fmt, ...)
 /* dump message msg and len bytes from buf to upsdebugx(level) in
    hexadecimal. (This function replaces Philippe Marzouk's original
    dump_hex() function) */
-void upsdebug_hex(int level, const char *msg, const unsigned char *buf, int len)
+void upsdebug_hex(int level, const char *msg, const void *buf, int len)
 {
 	char line[100];
-	int n = 0; /* number of characters currently in line */
-	int i = 0; /* number of bytes output from buffer */
+	int n;	/* number of characters currently in line */
+	int i;	/* number of bytes output from buffer */
 
-	n += snprintf(line, 100, "%s: (%d bytes) =>", msg, len);
-	while (i < len) {
-		if (n+3 > 75) {
+	n = snprintf(line, sizeof(line), "%s: (%d bytes) =>", msg, len); 
+
+	for (i = 0; i < len; i++) {
+
+		if (n > 72) {
 			upsdebugx(level, "%s", line);
 			line[0] = 0;
-			n = 0;
 		}
-		n += sprintf(line+n, n ? " %02x" : "%02x", buf[i]);
-		i++;
+
+		n = snprintfcat(line, sizeof(line), n ? " %02x" : "%02x",
+			((unsigned char *)buf)[i]);
 	}
 	upsdebugx(level, "%s", line);
 }
@@ -470,3 +472,51 @@ char *rtrim(char *in, const char sep)
 	
 	return in;
 }
+
+/* Read up to buflen bytes from fd and return the number of bytes
+   read. If no data is available within d_sec + d_usec, return 0.
+   On error, a value < 0 is returned (errno indicates error). */
+int select_read(const int fd, void *buf, const size_t buflen, const long d_sec, const long d_usec)
+{
+	int		ret;
+	fd_set		fds;
+	struct timeval	tv;
+
+	FD_ZERO(&fds);
+	FD_SET(fd, &fds);
+
+	tv.tv_sec = d_sec;
+	tv.tv_usec = d_usec;
+
+	ret = select(fd + 1, &fds, NULL, NULL, &tv);
+
+	if (ret < 1) {
+		return ret;
+	}
+
+	return read(fd, buf, buflen);
+}
+
+/* Write up to buflen bytes to fd and return the number of bytes
+   written. If no data is available within d_sec + d_usec, return 0.
+   On error, a value < 0 is returned (errno indicates error). */
+int select_write(const int fd, const void *buf, const size_t buflen, const long d_sec, const long d_usec)
+{
+	int		ret;
+	fd_set		fds;
+	struct timeval	tv;
+
+	FD_ZERO(&fds);
+	FD_SET(fd, &fds);
+
+	tv.tv_sec = d_sec;
+	tv.tv_usec = d_usec;
+
+	ret = select(fd + 1, NULL, &fds, NULL, &tv);
+
+	if (ret < 1) {
+		return ret;
+	}
+
+	return write(fd, buf, buflen);
+}
