@@ -338,9 +338,11 @@ static void do_capabilities(void)
 			if (quirk_capability_overflow)
 				return;
 
-			printf("Capability string has overflowed\n");
-			printf("Please report this error\n");
-			fatalx("ERROR: capability overflow!");
+			fatalx(EXIT_FAILURE, 
+				"Capability string has overflowed\n"
+				"Please report this error\n"
+				"ERROR: capability overflow!"
+				);
 		}
 
 		cmd = ptr[0];
@@ -532,11 +534,10 @@ static int firmware_table_lookup(void)
 			SER_WAIT_SEC, SER_WAIT_USEC);
 
 		/* found one, force the model information */
-		if (!strcmp(buf, "6QD")) {
-			upsdebugx(1, "Found Smart-UPS");
-			dstate_setinfo("ups.model", "Smart-UPS");
-		}
-		else if (!strcmp(buf, "6TI")) {
+		if (!strcmp(buf, "6QD") || /* (APC600.) */
+				!strcmp(buf, "8QD") || /* (SmartUPS 1250, vintage 07/94.) */
+				!strcmp(buf, "6TI") || /* (APC600.) */
+				!strcmp(buf, "6QI")) { /* (APC600.) */
 			upsdebugx(1, "Found Smart-UPS");
 			dstate_setinfo("ups.model", "Smart-UPS");
 		}
@@ -610,17 +611,13 @@ static void getbaseinfo(void)
 	*/
  	alrts = strchr(temp, '.');
 	if (alrts == NULL) {
-		printf("Unable to split APC version string\n");
-		printf("Bailing out\n");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, "Unable to split APC version string");
 	}
 	*alrts++ = 0;
 
 	cmds = strchr(alrts, '.');
 	if (cmds == NULL) {
-		printf("Unable to find APC command string\n");
-		printf("Bailing out\n");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, "Unable to find APC command string");
 	}
 	*cmds++ = 0;
 
@@ -851,11 +848,8 @@ void upsdrv_shutdown(void)
 /* 940-0095B support: set DTR, lower RTS */
 static void init_serial_0095B(void)
 {
-	int	dtr_bit = TIOCM_DTR;
-	int	rts_bit = TIOCM_RTS;
-
-	ioctl(upsfd, TIOCMBIS, &dtr_bit);
-	ioctl(upsfd, TIOCMBIC, &rts_bit);
+	ser_set_dtr(upsfd, 1);
+	ser_set_rts(upsfd, 0);
 }
 
 static void update_info_normal(void)
@@ -1245,10 +1239,10 @@ void upsdrv_help(void)
 void upsdrv_initinfo(void)
 {
 	if (!smartmode()) {
-		printf("Unable to detect an APC Smart protocol UPS on port %s\n", 
-			device_path);
-		printf("Check the cabling, port name or model name and try again\n");
-		exit(EXIT_FAILURE);
+		fatalx(EXIT_FAILURE, 
+			"Unable to detect an APC Smart protocol UPS on port %s\n"
+			"Check the cabling, port name or model name and try again", device_path
+			);
 	}
 
 	/* manufacturer ID - hardcoded in this particular module */
