@@ -254,12 +254,15 @@ struct {
 	/* APC600 */
 	{ "6QD",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
 	{ "6TI",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
+	{ "6QI",        "79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
 	/* CS 350 */
 	{ "5.4.D",	"\1ABPQRSUYbdfgjmnx9",	0 },
 	/* Smart-UPS 600 */
 	{ "D6",         "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
 	{ "D5",         "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
 	{ "D4",         "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
+	/* SmartUPS 1250. */
+	{ "8QD",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
 
 	{ NULL,		NULL,			0 },
 };
