@@ -28,6 +28,7 @@
 #include <sys/un.h>
 
 #include "common.h"
+#include "config.h"
 #include "dstate-hal.h"
 #include "extstate.h"
 /* #include "state.h"
@@ -85,11 +86,14 @@ struct	ups_handler	upsh;
 
 LibHalContext *halctx = NULL;
 char *udi;
-
 int ac_present = 0; /* 0 = false ; 1 = true */
-static void* runtime_handler(LibHalChangeSet *cs, char* runtime);
+extern char *dbus_methods_introspection;
 
-/* Structure to lookup between NUT and HAL */
+static void* runtime_handler(LibHalChangeSet *cs, char* runtime);
+static void* level_handler(LibHalChangeSet *cs, char* critical_level);
+static void* battery_type_handler(LibHalChangeSet *cs, char* battery_type);
+
+/* Structure to lookup between NUT and HAL data */
 typedef struct {
 	char	*nut_name;				/* NUT variable name */
 	char	*hal_name;				/* HAL variable name */
@@ -106,19 +110,51 @@ enum hal_type_t
 	HAL_TYPE_STRING
 };
 
-/* Data to lookup between NUT and HAL for dstate_setinfo() */
-static info_lkp_t nut2hal[] =
+/* Structure to lookup between NUT commands and HAL/DBus methods */
+typedef struct {
+	char	*nut_name;				/* NUT command name */
+	char	*hal_name;				/* HAL/DBus method name */
+	char	*xml_introspection;		/* HAL/DBus method introspection */
+/* FIXME: how to lookup param values between HAL and NUT?? */
+/*	void    *(*fun)(LibHalChangeSet *cs, 
+					char *value);*/	/* NUT function */
+} method_lkp_t;
+
+/* Data to lookup between NUT commands and HAL/DBus methods
+ * for dstate_addcmd() */
+static method_lkp_t nut2hal_cmd[] =
 {
 	/* ups.status is handled by status_set() calls */
-	{ "battery.charge.low", "battery.charge_level.low", HAL_TYPE_INT, NULL },
-	{ "battery.charge.low", "battery.reporting.low", HAL_TYPE_INT, NULL },
+	{
+		"beeper.enable",
+		"SetBeeper",
+		"    <method name=\"SetBeeper\">\n"
+		"      <arg name=\"beeper_mode\" direction=\"in\" type=\"b\"/>\n"
+		"      <arg name=\"return_code\" direction=\"out\" type=\"i\"/>\n"
+		"    </method>\n"
+ 	},
+ 
+	/* Terminating element */
+	{ NULL, NULL, NULL }
+};
+
+/* Data to lookup between NUT and HAL for dstate_setinfo() */
+static info_lkp_t nut2hal_info[] =
+{
+	/* ups.status is handled by status_set() calls */
+	{ "battery.charge.low", "battery.charge_level.low", HAL_TYPE_INT, *level_handler },
+	/* { "battery.charge.low", "battery.reporting.low", HAL_TYPE_INT, NULL }, */
+	{ "battery.charge.low", "battery.alarm.design", HAL_TYPE_INT, NULL },
+	
 	{ "battery.charge", "battery.charge_level.current", HAL_TYPE_INT, NULL },
 	{ "battery.charge", "battery.charge_level.percentage", HAL_TYPE_INT, NULL },
 	{ "battery.charge", "battery.reporting.current", HAL_TYPE_INT, NULL },
 	{ "battery.charge", "battery.reporting.percentage", HAL_TYPE_INT, NULL },
 	{ "battery.runtime", "battery.remaining_time", HAL_TYPE_INT, *runtime_handler },
-	{ "battery.type", "battery.technology", HAL_TYPE_STRING, NULL },
+	/* raw version (PbAc) */
 	{ "battery.type", "battery.reporting.technology", HAL_TYPE_STRING, NULL },
+	/* Human readable version */
+	{ "battery.type", "battery.technology", HAL_TYPE_STRING, *battery_type_handler },
 	
 	/* AQ note: Not sure it fits!	*/
 	/* HAL marked as mandatory! */
@@ -140,6 +176,28 @@ static info_lkp_t *find_nut_info(const char *nut_varname, info_lkp_t *prev_info_
 void hal_set_string(LibHalChangeSet *cs, const char *key, const char *value);
 void hal_set_int(LibHalChangeSet *cs, const char *key, const int value);
 void hal_set_bool(LibHalChangeSet *cs, const char *key, const dbus_bool_t value);
+int hal_get_int(const char *key);
+char *hal_get_string(const char *key);
+
+/* Handle warning charge level according to the critical level */
+static void* level_handler(LibHalChangeSet *cs, char* critical_level)
+{
+	/* Magic formula to generate the warning level */
+	int int_critical_level = atoi(critical_level);
+	/* warning level = critical + 1/3 of (100 % - critical level), approx at the leat mod 10 */
+	int int_warning_level = int_critical_level + ((100 - int_critical_level) / 3);
+	int_warning_level -= (int_warning_level % 10);
+
+	/* Set the critical level value */
+	hal_set_int (cs, "battery.charge_level.low", int_critical_level);
+	hal_set_int (cs, "battery.reporting.low", int_critical_level);
+	
+	/* Set the warning level value (FIXME: set to 50 % for now) */
+	hal_set_int (cs, "battery.charge_level.warning", int_warning_level);
+	hal_set_int (cs, "battery.reporting.warning", int_warning_level);
+
+	return NULL; /* Nothing to return */
+}
 
 /* Handle runtime exposition according to the AC status */
 static void* runtime_handler(LibHalChangeSet *cs, char* runtime)
@@ -161,6 +219,17 @@ static void* runtime_handler(LibHalChangeSet *cs, char* runtime)
 	return NULL; /* Nothing to return */
 }
 
+/* Handle the battery technology reporting */
+static void* battery_type_handler(LibHalChangeSet *cs, char* battery_type)
+{
+	if (!strncmp (battery_type, "PbAc", 4)) {
+		hal_set_string(cs, "battery.technology", "lead-acid");
+	}
+	/* FIXME: manage other types (lithium-ion, lithium-polymer,
+	 *  nickel-metal-hydride, unknown */
+
+	return NULL; /* Nothing to return */
+}
 
 /********************************************************************
  * dstate compatibility interface
@@ -180,6 +249,7 @@ void dstate_init(const char *prog, const char *port)
 	/* UPS always report charge as percent */
 	hal_set_string (cs, "battery.charge_level.unit", "percent");
 	hal_set_string (cs, "battery.reporting.unit", "percent");
+	hal_set_string (cs, "battery.alarm.unit", "percent");
 
 	/* Various UPSs assumptions */
 	/****************************/
@@ -215,11 +285,12 @@ void dstate_init(const char *prog, const char *port)
 	libhal_device_add_capability (halctx, udi, "battery", &dbus_error);
 	libhal_device_add_capability (halctx, udi, "ac_adaptor", &dbus_error);
 	
-	/* FIXME: what's that? (from addon-hidups) */
-	/* UPS_DEVICENAME
-	libhal_changeset_set_property_string (
-	cs, "foo", ups_get_string (fd, uref.value)); */
-
+	/* FIXME: can be improved?! Set granularity (1 %)*/
+	hal_set_int (cs, "battery.charge_level.granularity_1", 1);
+	hal_set_int (cs, "battery.charge_level.granularity_2", 1);
+	hal_set_int (cs, "battery.reporting.granularity_1", 1);
+	hal_set_int (cs, "battery.reporting.granularity_2", 1);
+	
 	dbus_error_init (&dbus_error);
 	/* NOTE: commit_changeset won't do IPC if set is empty */
 	libhal_device_commit_changeset (halctx, cs, &dbus_error);
@@ -232,9 +303,31 @@ void dstate_init(const char *prog, const char *port)
 
 const char *dstate_getinfo(const char *var)
 {
-	/* FIXME: use revert lookup from nut2hal_info or
-	from raw NUT info? */
-	return NULL;
+	char *value = NULL;
+	int ivalue;
+	info_lkp_t *nut2hal_info = find_nut_info(var, NULL);
+
+/* FIXME: use the data exposed by NUT on the DBus when available */
+	
+	if (nut2hal_info != NULL) {
+
+		switch (nut2hal_info->hal_type)
+		{
+/*			case HAL_TYPE_INT:
+				ivalue = hal_get_int(nut2hal_info->hal_name);
+				value = xstrdup("12345");
+				snprintf(value, 5, "%i", ivalue);
+				break;
+			case HAL_TYPE_BOOL:
+				hal_set_bool(cs, nut2hal_info->hal_name, TRUE);
+				break;
+*/
+			case HAL_TYPE_STRING:
+				value = hal_get_string(nut2hal_info->hal_name);
+				break;
+		}
+	}	
+	return value;
 }
 
 int dstate_setinfo(const char *var, const char *fmt, ...)
@@ -329,8 +422,6 @@ void dstate_free(void)
 /* extrafd: provided for waking up based on the driver's UPS fd */
 int dstate_poll_fds(int interval, int extrafd)
 {
-	/* FIXME: can it be enhanced? */
-	sleep (interval);
 	return 0;
 }	
 
@@ -367,6 +458,7 @@ void status_commit(void)
 {
 	LibHalChangeSet *cs;
 	DBusError dbus_error;
+	int curlevel, warnlevel, lowlevel;
 
 	upsdebugx(2, "status_commit");
 
@@ -375,6 +467,11 @@ void status_commit(void)
 		fatalx (EXIT_FAILURE, "Cannot initialize changeset");
 	}
 
+	/* Retrieve the levels */
+	curlevel = hal_get_int ("battery.charge_level.current");
+	warnlevel = hal_get_int ("battery.charge_level.warning");
+	lowlevel = hal_get_int ("battery.charge_level.low");
+
 	/* Set AC present status */
 	/* Note: UPSs are also AC adaptors! */
 	hal_set_bool (cs, "ac_adaptor.present", (ac_present == 0)?FALSE:TRUE);
@@ -383,7 +480,16 @@ void status_commit(void)
 	hal_set_bool (cs, "battery.rechargeable.is_discharging", (ac_present == 0)?TRUE:FALSE);
 
 	/* Set charging status */
-	hal_set_bool (cs, "battery.rechargeable.is_charging", (ac_present == 0)?FALSE:TRUE);
+	if (curlevel != 100)
+		hal_set_bool (cs, "battery.rechargeable.is_charging", (ac_present == 0)?FALSE:TRUE);
+
+	/* Set the battery status (FIXME: are these values valid?) */
+	if (curlevel <= lowlevel)
+		hal_set_string(cs, "battery.charge_level.capacity_state", "critical");
+	else if (curlevel <= warnlevel)
+		hal_set_string(cs, "battery.charge_level.capacity_state", "warning"); /*low?*/
+	else
+		hal_set_string(cs, "battery.charge_level.capacity_state", "ok");
 
 	dbus_error_init (&dbus_error);
 	/* NOTE: commit_changeset won't do IPC if set is empty */
@@ -412,7 +518,8 @@ void alarm_commit(void)
 	return;
 }
 
-/* Register DBus methods */
+/* FIXME: complete and use nut2hal_cmd */
+/* Register DBus methods, by feeling the methods buffer */
 void dstate_addcmd(const char *cmdname)
 {
 	DBusError dbus_error;
@@ -421,139 +528,19 @@ void dstate_addcmd(const char *cmdname)
 	upsdebugx(2, "dstate_addcmd: %s\n", cmdname);
 
 	/* beeper.{on,off} */
-/*	if (!strncasecmp(cmdname, "beeper.o", 8))
+	/* FIXME: how to deal with beeper.toggle */
+	if (!strncasecmp(cmdname, "beeper.disable", 14))
 	{
-		if (!libhal_device_claim_interface(halctx, udi, DBUS_INTERFACE,
-			"    <method name=\"SetBeeper\">\n"
-			"      <arg name=\"beeper_mode\" direction=\"in\" type=\"b\"/>\n"
-			"      <arg name=\"return_code\" direction=\"out\" type=\"i\"/>\n"
-			"    </method>\n",
-			&dbus_error)) {
-				fprintf(stderr, "Error: can't claim DBus interface: %s", dbus_error.message);
-				exit(EXIT_FAILURE);
-		}
-	}
-
-	dbus_connection_setup_with_g_main(dbus_connection, NULL);
-	dbus_connection_add_filter(dbus_connection, dbus_filter_function, NULL, NULL);
-	dbus_connection_set_exit_on_disconnect(dbus_connection, 0);
-*/
-/*
-	if (!libhal_device_claim_interface(halctx, udi, DBUS_INTERFACE,
-		"    <method name=\"Shutdown\">\n"
-		"      <arg name=\"shutdown_type\" direction=\"in\" type=\"s\"/>\n"
+		strcat(dbus_methods_introspection,
+		"    <method name=\"SetBeeper\">\n"
+		"      <arg name=\"beeper_mode\" direction=\"in\" type=\"b\"/>\n"
 		"      <arg name=\"return_code\" direction=\"out\" type=\"i\"/>\n"
-		"    </method>\n",
-		&dbus_error)) {
-			fprintf(stderr, "Error: can't claim DBus interface: %s", dbus_error.message);
-			exit(EXIT_FAILURE);
+		"    </method>\n");
 	}
-*/
-	
-/*	if (dbus_error_is_set (&dbus_error))
-		dbus_error_free (&dbus_error); */
-}
-
-/********************************************************************
- * DBus interface functions and data
- *******************************************************************/
-
-
-static DBusHandlerResult dbus_filter_function_local(DBusConnection *connection,
-						    DBusMessage *message,
-						    void *user_data)
-{
-	if (dbus_message_is_signal(message, DBUS_INTERFACE_LOCAL, "Disconnected")) {
-		upsdebugx(1, "DBus daemon disconnected. Trying to reconnect...");
-		dbus_connection_unref(connection);
-		g_timeout_add(5000, (GSourceFunc)dbus_init_local, NULL);
-	}
-	return DBUS_HANDLER_RESULT_HANDLED;
-}
-
-/* returns FALSE on success because it's used as a callback */
-gboolean dbus_init_local(void)
-{
-	DBusConnection	*dbus_connection;
-	DBusError	dbus_error;
-
-	dbus_error_init(&dbus_error);
-
-	dbus_connection = dbus_bus_get(DBUS_BUS_SYSTEM, &dbus_error);
-	if (dbus_error_is_set(&dbus_error)) {
-		upsdebugx(1, "Cannot get D-Bus connection");
-/*		dbus_error_free (&dbus_error); */
-		return TRUE;
-	}
-
-	dbus_connection_setup_with_g_main(dbus_connection, NULL);
-	dbus_connection_add_filter(dbus_connection, dbus_filter_function_local,
-				   NULL, NULL);
-	dbus_connection_set_exit_on_disconnect(dbus_connection, 0);
-	return FALSE;
 }
 
 
-/** 
- * dbus_filter_function:
- * @connection:		connection to D-Bus
- * @message:		message
- * @user_data:  pointer to the data
- *
- * Returns: 		the result
- * 
- * @raises UnknownMethod
- *
- * D-Bus filter function
- */
-DBusHandlerResult dbus_filter_function(DBusConnection *connection,
-					      DBusMessage *message,
-					      void *user_data)
-{
-	DBusError dbus_error;
-	DBusConnection	*dbus_connection;
-	const char	*member		= dbus_message_get_member(message);
-	const char	*path		= dbus_message_get_path(message);
 
-	upsdebugx(2, "Received DBus message with\n\tmember %s\n\tpath %s", member, path);
-exit(0);
-
-	dbus_error_init(&dbus_error);
-	if (dbus_error_is_set (&dbus_error))
-   	{
-		fprintf (stderr, "an error occurred: %s\n", dbus_error.message);
-/*		dbus_error_free (&dbus_error); */
-	}
-	else
-	{
-#ifdef HAVE_POLKIT
-		if (!dbus_is_privileged(connection, message, &dbus_error))
-			return DBUS_HANDLER_RESULT_HANDLED;
-#endif
-
-		if (dbus_message_is_method_call(message, DBUS_INTERFACE,
-						"Shutdown")) {
-			char *arg;
-/*
-			if (!dbus_get_argument(connection, message, &dbus_error,
-					       DBUS_TYPE_STRING, &arg)) {
-				return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-			}
-	 		HAL_DEBUG(("Received argument: %s", arg));
-				
-			if (set_governors(connection, message, arg))
-				dbus_send_reply(connection, message, DBUS_TYPE_INVALID, NULL);
-
-		}
-*/
-		} else {
-			return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-		}
-	}
-	return DBUS_HANDLER_RESULT_HANDLED;
-}
-
-	
 /*******************************************************************
  * internal functions
  *******************************************************************/
@@ -615,6 +602,42 @@ void hal_set_int(LibHalChangeSet *cs, const char *key, const int value)
 	}	
 }
 
+char *hal_get_string(const char *key)
+{
+	DBusError dbus_error;
+	char *value = NULL;
+
+	upsdebugx(2, "hal_get_string: %s", key);
+
+	dbus_error_init(&dbus_error);
+
+	/* Check if the property already exists */
+	if (libhal_device_property_exists (halctx, udi, key, &dbus_error) == TRUE) {
+		
+		value = libhal_device_get_property_string (halctx, udi,
+				key, &dbus_error);
+	}
+	return value;
+}
+
+int hal_get_int(const char *key)
+{
+	DBusError dbus_error;
+	int value = -1;
+
+	upsdebugx(2, "hal_get_int: %s", key);
+
+	dbus_error_init(&dbus_error);
+
+	/* Check if the property already exists */
+	if (libhal_device_property_exists (halctx, udi, key, &dbus_error) == TRUE) {
+		
+		value = libhal_device_get_property_int (halctx, udi,
+				key, &dbus_error);
+	}
+	return value;
+}
+
 /* Only update HAL int values if there are real changes */
 void hal_set_bool(LibHalChangeSet *cs, const char *key, const dbus_bool_t value)
 {
@@ -656,7 +679,7 @@ static info_lkp_t *find_nut_info(const char *nut_varname, info_lkp_t *prev_info_
 		info_item = ++prev_info_item;
 	}
 	else {
-		info_item = nut2hal;
+		info_item = nut2hal_info;
 	}
 
 	for ( ; info_item != NULL && info_item->nut_name != NULL ; info_item++) {
