@@ -50,7 +50,6 @@
       correctly connected to the serial port).
  */
 
-#include <sys/ioctl.h>
 #include "main.h"
 #include "serial.h"
 #include "etapro.h"
@@ -66,7 +65,7 @@ etapro_get_response(const char *resp_type)
 	   Unlike ser_get_line(), don't discard the following characters
 	   because we have to handle multi-line responses.  */
 	n = 0;
-	while (ser_get_char(upsfd, &tmp[n], 1, 0) == 1) {
+	while (ser_get_char(upsfd, (unsigned char *)&tmp[n], 1, 0) == 1) {
 		if (n >= sizeof(tmp) - 1 || tmp[n] == '\n')
 			break;
 		n++;
@@ -352,14 +351,11 @@ upsdrv_banner(void)
 void
 upsdrv_initups(void)
 {
-	int dtr_bit = TIOCM_DTR;
-	int rts_bit = TIOCM_RTS;
-
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B1200);
 
-	ioctl(upsfd, TIOCMBIC, &dtr_bit);
-	ioctl(upsfd, TIOCMBIS, &rts_bit);
+	ser_set_dtr(upsfd, 0);
+	ser_set_rts(upsfd, 1);
 }
 
 void upsdrv_cleanup(void)
