@@ -20,14 +20,13 @@
 #include "main.h"
 #include "serial.h"
 #include "everups.h"
-#include <sys/ioctl.h>
 
 static	unsigned char	upstype = 0;
 
 static void init_serial(void)
 {
-        int     clr_bit = TIOCM_DTR | TIOCM_RTS;
-        ioctl(upsfd, TIOCMBIC, &clr_bit);
+	ser_set_dtr(upsfd, 0);
+	ser_set_rts(upsfd, 0);
 }
 
 static int Code(int tries)
@@ -154,13 +153,13 @@ void upsdrv_updateinfo(void)
 void upsdrv_shutdown(void)
 {
 	if (!Code(2)) {
-		upslogx(LOG_INFO, "Code failed: %m\n");
+		upslog_with_errno(LOG_INFO, "Code failed");
 		return;
 	}
 	ser_send_char(upsfd, 28);
 	ser_send_char(upsfd, 1);  /* 1.28 sec */
 	if (!Code(1)) {
-		upslogx(LOG_INFO, "Code failed: %m\n");
+		upslog_with_errno(LOG_INFO, "Code failed");
 		return;
 	}
 	ser_send_char(upsfd, 13);
