@@ -24,7 +24,6 @@
 #include "main.h"
 #include "serial.h"
 #include "liebert.h"
-#include <sys/ioctl.h>
 
 #define	ML_ONBATTERY	0x55
 
@@ -34,7 +33,7 @@ void upsdrv_shutdown(void)
 
 	/* worse yet: stock cables don't support shutdown at all */
 
-	fatalx("shutdown not supported");
+	fatalx(EXIT_FAILURE, "shutdown not supported");
 }
 
 void upsdrv_initinfo(void)
@@ -62,8 +61,7 @@ void upsdrv_initinfo(void)
 /* normal idle loop - keep up with the current state of the UPS */
 void upsdrv_updateinfo(void)
 {
-	int	flags, ret;
-	char	c;
+	unsigned char	c;
 	unsigned int	ob, lb;
 	static	unsigned int	ob_state = 0, ob_last = 0, ob_ctr = 0;
 	static	unsigned int	lb_state = 0, lb_last = 0, lb_ctr = 0;
@@ -72,24 +70,18 @@ void upsdrv_updateinfo(void)
 
 	/* the UPS connects RX to TX when on battery, so test for loopback */
 
+	ser_flush_in(upsfd, "", 0);
+
 	c = ML_ONBATTERY;
-	write(upsfd, &c, 1);
-	if (read(upsfd, &c, 1) == 1) {
-		while (read(upsfd, &c, 1) == 1)
+	ser_send_char(upsfd, c);
+	if (ser_get_char(upsfd, &c, 1, 0) == 1) {
+		while (ser_get_char(upsfd, &c, 1, 0) == 1)
 			continue;
 		if (c == ML_ONBATTERY)
 			ob = 1;
 	}
 	
-	ret = ioctl(upsfd, TIOCMGET, &flags);
-
-	if (ret != 0) {
-		upslog_with_errno(LOG_INFO, "ioctl failed");
-		dstate_datastale();
-		return;
-	}
-
-	if (flags & TIOCM_CD)
+	if (ser_get_dcd(upsfd))
 		lb = 1;
 
 	/* state machine below to ensure status changes are debounced */
@@ -178,12 +170,10 @@ void upsdrv_help(void)
 
 void upsdrv_initups(void)
 {
-	int rts_bit = TIOCM_RTS;
-
 	upsfd = ser_open(device_path);
 
 	/* raise RTS */
-	ioctl(upsfd, TIOCMBIS, &rts_bit);
+	ser_set_rts(upsfd, 1);
 }
 
 void upsdrv_cleanup(void)
