@@ -53,6 +53,7 @@
 
 #define SEND_PACE    100000  /* interval between chars on send (usec) */
 #define READ_TIMEOUT 2       /* timeout on read (seconds) */
+#define READ_PACE    300000  /* interval to wait between sending a command and reading the response (usec) */
 
 #define MAX_START_DELAY    9999
 #define MAX_SHUTDOWN_DELAY 99
@@ -61,6 +62,8 @@
 #define MAX_START_DELAY_LEN    4
 #define MAX_SHUTDOWN_DELAY_LEN 2
 
+#define MAX_POLL_FAILURES 3
+
 #define N_FLAGS 8
 
 /* The UPS status flags */
@@ -78,6 +81,9 @@
 #define UPS_MODEL_CHARS   10
 #define UPS_VERSION_CHARS 10
 
+/* Below this value we can safely consider a voltage to be zero */
+#define RESIDUAL_VOLTAGE 10.0
+
 
 /* The values returned by the UPS for an "I" query */
 typedef struct {
@@ -132,6 +138,10 @@ static BatteryVolts_t batteries[] = {{ 12.0,  9.0, 16.0,  9.7, 13.7,  0.0 },   /
                                      { 96.0,  1.5,  3.0, 1.63, 2.29,  1.8 },   /* Ablerex MS3000RT (LB at 25% charge) */
                                      {  0.0,  0.0,  0.0,  0.0,  0.0,  0.0 }};  /* END OF DATA */
 
+
+/* Workaround for buggy models */
+static char ignore_off = 0;  /* ignore FL_LOAD_OFF if it behaves strangely */
+
 /* Defined in upsdrv_initinfo */
 static float battvolt_empty = -1;  /* unknown */
 static float battvolt_full = -1;   /* unknown */
@@ -277,7 +287,10 @@ static int get_ups_info(UPSInfo_t *info)
 	int ret;
 
 	upsdebugx(2, "Asking for UPS information [I]...");
+	ser_flush_io(upsfd);
 	ser_send_pace(upsfd, SEND_PACE, "I%c", ENDCHAR);
+	usleep(READ_PACE);
+
 	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
 
 	if (ret < 0) {
@@ -323,7 +336,10 @@ static int get_firmware_values(FirmwareValues_t *values)
 	int ret;
 
 	upsdebugx(2, "Asking for UPS power ratings [F]...");
+	ser_flush_io(upsfd);
 	ser_send_pace(upsfd, SEND_PACE, "F%c", ENDCHAR);
+	usleep(READ_PACE);
+
 	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
 
 	if (ret < 0) {
@@ -363,7 +379,10 @@ static int run_query(QueryValues_t *values)
 	int ret;
 
 	upsdebugx(2, "Asking for UPS status [Q1]...");
+	ser_flush_io(upsfd);
 	ser_send_pace(upsfd, SEND_PACE, "Q1%c", ENDCHAR);
+	usleep(READ_PACE);
+
 	ret = ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0);
 
 	if (ret < 0) {
@@ -397,7 +416,7 @@ static int run_query(QueryValues_t *values)
 	 * temperature reading available.
 	 */
 	values->temp = atof(temperature);
-
+	
 	return 0;
 }
 
@@ -433,6 +452,11 @@ void upsdrv_initinfo(void)
 		}
 	}
 
+	dstate_setinfo("ups.type", status.flags[FL_UPS_TYPE] == '1' ? "standby" : "online");
+
+	upsdebugx(1, "Cancelling any pending shutdown or battery test.");
+	ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
+
 	/*
 	 * Try to identify the UPS.
 	 */
@@ -453,12 +477,24 @@ void upsdrv_initinfo(void)
 
 	dstate_setinfo("ups.serial", "%s", getval("serial") ? getval("serial") : "unknown");
 
+	/*
+	 * Workaround for buggy models.
+	 */
+	ignore_off = testvar("ignoreoff");
+
+	if (status.flags[FL_LOAD_OFF] == '1' && status.load > 0.01 && !ignore_off) {
+		ignore_off = 1;
+		upslogx(LOG_INFO, "The UPS reports OFF status but appears to be ON. Parameter \"ignoreoff\" set automatically.");
+	}
+	upsdebugx(2, "Parameter [ignoreoff]: [%s]", (ignore_off ? "true" : "false"));
+
 	/*
 	 * Set battery-related values.
 	 */
 	if (get_firmware_values(&values) >= 0) {
-		dstate_setinfo("output.voltage.nominal", "%.1f", values.volt);
 		dstate_setinfo("battery.voltage.nominal", "%.1f", values.battvolt);
+		dstate_setinfo("input.voltage.nominal", "%.1f", values.volt);
+		dstate_setinfo("input.frequency.nominal", "%.1f", values.freq);
 
 		if (set_battery_params(values.battvolt, status.battvolt) < 0) {
 			upslogx(LOG_NOTICE, "This UPS has an unsupported combination of battery voltage/number of batteries.");
@@ -472,7 +508,7 @@ void upsdrv_initinfo(void)
 			fatalx(EXIT_FAILURE, "Error in \"battvolts\" parameter.");
 		}
 
-	    upslogx(LOG_NOTICE, "Overriding battery voltage interval [%.1fV, %.1fV].", battvolt_empty, battvolt_full);
+		upslogx(LOG_NOTICE, "Overriding battery voltage interval [%.1fV, %.1fV].", battvolt_empty, battvolt_full);
 	}
 
 	if (battvolt_empty < 0 || battvolt_full < 0) {
@@ -519,9 +555,6 @@ void upsdrv_initinfo(void)
 	upsh.instcmd = instcmd;
 	upsh.setvar = setvar;
 
-	/* clean up a possible shutdown in progress */
-	ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
-
 	upsdebugx(1, "Done setting up the UPS.");
 }
 
@@ -530,6 +563,7 @@ void upsdrv_updateinfo(void)
 {
 	QueryValues_t query;
 	float charge;
+	static int poll_fail = 0;
 
 	if (run_query(&query) < 0) {
 		/*
@@ -540,18 +574,31 @@ void upsdrv_updateinfo(void)
 		 * Some fault tolerance is good, we just assume
 		 * that the UPS is just taking a nap. ;)
 		 */
-		dstate_datastale();
+		poll_fail++;
+		upsdebugx(2, "Poll failure [%d].", poll_fail);		
+		ser_comm_fail("No status from UPS.");
+
+		if (poll_fail >= MAX_POLL_FAILURES) {
+			upsdebugx(2, "Too many poll failures, data is stale.");
+			dstate_datastale();
+		}
 
 		return;
 	}
 
+	poll_fail = 0;
+	ser_comm_good();
+
 	dstate_setinfo("input.voltage", "%.1f", query.ivolt);
 	dstate_setinfo("input.voltage.fault", "%.1f", query.fvolt);
 	dstate_setinfo("output.voltage", "%.1f", query.ovolt);
 	dstate_setinfo("ups.load", "%.1f", query.load);
 	dstate_setinfo("input.frequency", "%.1f", query.freq);
 	dstate_setinfo("battery.voltage", "%.2f", query.battvolt);
-	dstate_setinfo("ups.temperature", "%.1f", query.temp);
+
+	if (query.temp > 0.01) {
+		dstate_setinfo("ups.temperature", "%.1f", query.temp);
+	}
 
 	charge = get_battery_charge(query.battvolt);
 	if (charge >= 0) {
@@ -560,11 +607,16 @@ void upsdrv_updateinfo(void)
 		upsdebugx(2, "Calculated battery charge: %.1f%%", charge);
 	}
 
+	dstate_setinfo("ups.beeper.status", query.flags[FL_BEEPER_ON] == '1' ? "enabled" : "disabled");
+
 	status_init();
 
-	if (query.flags[FL_LOAD_OFF] == '1') {
-		status_set("OFF");
-	} else if (query.flags[FL_ON_BATT] == '1' || query.flags[FL_BATT_TEST] == '1') {
+	/*
+	 * Some models, when OFF, never change to on-battery status when
+	 * line power is unavailable. To get around this, we also look at
+	 * the input voltage level here.
+	 */
+	if (query.flags[FL_ON_BATT] == '1' || query.ivolt < RESIDUAL_VOLTAGE) {
 		status_set("OB");
 	} else {
 		status_set("OL");
@@ -578,27 +630,8 @@ void upsdrv_updateinfo(void)
 				status_set("BYPASS");
 			}
 		}
-	}
 
-	/*
-	 * If "lowbatt > 0", it becomes a "soft" low battery level
-	 * and the hardware flag "FL_LOW_BATT" is always ignored.
-	 */
-	if ((lowbatt <= 0 && query.flags[FL_LOW_BATT] == '1') ||
-	    (lowbatt > 0 && charge < lowbatt)) {
-		status_set("LB");
-	}
-
-	if (query.flags[FL_FAILED] == '1') {
-		status_set("FAILED");
-	}
-
-	status_commit();
-
-	dstate_setinfo("ups.beeper.status", query.flags[FL_BEEPER_ON] == '1' ? "enabled" : "disabled");
-
-	/* Update minimum and maximum input voltage levels only when on line */
-	if (query.flags[FL_ON_BATT] == '0') {
+		/* Update minimum and maximum input voltage levels too */
 		if (query.ivolt < ivolt_min) {
 			ivolt_min = query.ivolt;
 		}
@@ -611,6 +644,33 @@ void upsdrv_updateinfo(void)
 		dstate_setinfo("input.voltage.maximum", "%.1f", ivolt_max);
 	}
 
+	/*
+	 * If "lowbatt > 0", it becomes a "soft" low battery level
+	 * and the hardware flag "FL_LOW_BATT" is always ignored.
+	 */
+	if ((lowbatt <= 0 && query.flags[FL_LOW_BATT] == '1') ||
+	    (lowbatt > 0 && charge < lowbatt)) {
+		status_set("LB");
+	}
+
+	if (query.flags[FL_BATT_TEST] == '1') {
+		status_set("CAL");
+	}
+
+	if (query.flags[FL_LOAD_OFF] == '1' && !ignore_off) {
+		status_set("OFF");
+	}
+
+	alarm_init();
+
+	if (query.flags[FL_FAILED] == '1') {
+		alarm_set("Internal UPS fault!");
+	}
+
+	alarm_commit();
+
+	status_commit();
+
 	dstate_dataok();
 }
 
@@ -634,15 +694,16 @@ int instcmd(const char *cmdname, const char *extra)
 	/*
 	 * Some commands are always supported by every UPS implementing
 	 * the megatec protocol, but others may or may not be supported.
-	 * For these we must check if the UPS returns something, which
-	 * would mean the command is unsupported.
+	 * Unsupported commands are echoed back without ENDCHAR.
 	 */
 
 	if (strcasecmp(cmdname, "test.battery.start.deep") == 0) {
+		ser_flush_io(upsfd);
 		ser_send_pace(upsfd, SEND_PACE, "TL%c", ENDCHAR);
+		usleep(READ_PACE);
 
-		if (ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0) > 0) {
-			upslogx(LOG_NOTICE, "test.battery.start.deep not supported by UPS.");
+		if (ser_get_line(upsfd, buffer, 2 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
+			upslogx(LOG_NOTICE, "test.battery.start.deep not supported by UPS hardware.");
 		} else {
 			upslogx(LOG_INFO, "Deep battery test started.");
 		}
@@ -651,10 +712,12 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (strcasecmp(cmdname, "test.battery.start") == 0) {
+		ser_flush_io(upsfd);
 		ser_send_pace(upsfd, SEND_PACE, "T%c", ENDCHAR);
+		usleep(READ_PACE);
 
-		if (ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0) > 0) {
-			upslogx(LOG_NOTICE, "test.battery.start not supported by UPS.");
+		if (ser_get_line(upsfd, buffer, 1 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
+			upslogx(LOG_NOTICE, "test.battery.start not supported by UPS hardware.");
 		} else {
 			upslogx(LOG_INFO, "Battery test started.");
 		}
@@ -663,10 +726,12 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (strcasecmp(cmdname, "test.battery.stop") == 0) {
+		ser_flush_io(upsfd);
 		ser_send_pace(upsfd, SEND_PACE, "CT%c", ENDCHAR);
+		usleep(READ_PACE);
 
-		if (ser_get_line(upsfd, buffer, RECV_BUFFER_LEN, ENDCHAR, IGNCHARS, READ_TIMEOUT, 0) > 0) {
-			upslogx(LOG_NOTICE, "test.battery.stop not supported by UPS.");
+		if (ser_get_line(upsfd, buffer, 2 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
+			upslogx(LOG_NOTICE, "test.battery.stop not supported by UPS hardware.");
 		} else {
 			upslogx(LOG_INFO, "Battery test stopped.");
 		}
@@ -689,7 +754,14 @@ int instcmd(const char *cmdname, const char *extra)
 		ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
 		watchdog_enabled = 0;
 
-		ser_send_pace(upsfd, SEND_PACE, "S%02dR0000%c", shutdown_delay, ENDCHAR);
+		ser_flush_io(upsfd);
+		ser_send_pace(upsfd, SEND_PACE, "S%02d%c", shutdown_delay, ENDCHAR);
+		usleep(READ_PACE);
+
+		if (ser_get_line(upsfd, buffer, 3 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
+			ser_send_pace(upsfd, SEND_PACE, "S%02dR9999%c", shutdown_delay, ENDCHAR);
+			upslogx(LOG_NOTICE, "UPS refuses to turn the load off indefinitely. Will turn off for 9999 minutes instead.");
+		}
 
 		upslogx(LOG_INFO, "Shutdown (stayoff) initiated.");
 
@@ -709,7 +781,7 @@ int instcmd(const char *cmdname, const char *extra)
 		ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
 		watchdog_enabled = 0;
 
-		upslogx(LOG_INFO, "Turning load on.");
+		upslogx(LOG_INFO, "Turning the load on.");
 
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -718,9 +790,16 @@ int instcmd(const char *cmdname, const char *extra)
 		ser_send_pace(upsfd, SEND_PACE, "C%c", ENDCHAR);
 		watchdog_enabled = 0;
 
-		ser_send_pace(upsfd, SEND_PACE, "S00R0000%c", ENDCHAR);
+		ser_flush_io(upsfd);
+		ser_send_pace(upsfd, SEND_PACE, "S00%c", ENDCHAR);
+		usleep(READ_PACE);
 
-		upslogx(LOG_INFO, "Turning load off.");
+		if (ser_get_line(upsfd, buffer, 3 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
+			ser_send_pace(upsfd, SEND_PACE, "S00R9999%c", ENDCHAR);
+			upslogx(LOG_NOTICE, "UPS refuses to turn the load off indefinitely. Will turn off for 9999 minutes instead.");
+		}
+
+		upslogx(LOG_INFO, "Turning the load off.");
 
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -752,9 +831,15 @@ int instcmd(const char *cmdname, const char *extra)
 	}
 
 	if (strcasecmp(cmdname, "beeper.toggle") == 0) {
+		ser_flush_io(upsfd);
 		ser_send_pace(upsfd, SEND_PACE, "Q%c", ENDCHAR);
+		usleep(READ_PACE);
 
-		upslogx(LOG_INFO, "Toggling UPS beeper.");
+		if (ser_get_line(upsfd, buffer, 1 + 1, '\0', IGNCHARS, READ_TIMEOUT, 0) > 0) {
+			upslogx(LOG_NOTICE, "beeper.toggle not supported by UPS hardware.");
+		} else {
+			upslogx(LOG_INFO, "Toggling UPS beeper.");
+		}
 
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -778,13 +863,14 @@ void upsdrv_help(void)
 
 void upsdrv_makevartable(void)
 {
-	addvar(VAR_VALUE, "mfr", "Manufacturer name");
-	addvar(VAR_VALUE, "model", "Model name");
-	addvar(VAR_VALUE, "serial", "UPS serial number");
-	addvar(VAR_VALUE, "lowbatt", "Low battery level (%)");
-	addvar(VAR_VALUE, "ondelay", "Minimum delay before UPS startup (minutes)");
-	addvar(VAR_VALUE, "offdelay", "Delay before UPS shutdown (minutes)");
+	addvar(VAR_VALUE, "mfr"      , "Manufacturer name");
+	addvar(VAR_VALUE, "model"    , "Model name");
+	addvar(VAR_VALUE, "serial"   , "UPS serial number");
+	addvar(VAR_VALUE, "lowbatt"  , "Low battery level (%)");
+	addvar(VAR_VALUE, "ondelay"  , "Minimum delay before UPS startup (minutes)");
+	addvar(VAR_VALUE, "offdelay" , "Delay before UPS shutdown (minutes)");
 	addvar(VAR_VALUE, "battvolts", "Battery voltages (empty:full)");
+	addvar(VAR_FLAG , "ignoreoff", "Ignore the OFF status reported by the UPS.");
 
 	megatec_subdrv_makevartable();
 }
@@ -793,7 +879,7 @@ void upsdrv_makevartable(void)
 void upsdrv_banner(void)
 {
 	printf("Network UPS Tools %s - Megatec protocol driver %s [%s]\n", UPS_VERSION, DRV_VERSION, progname);
-	printf("Carlos Rodrigues (c) 2003-2007\n\n");
+	printf("Carlos Rodrigues (c) 2003-2008\n\n");
 
 	megatec_subdrv_banner();
 }
