@@ -24,61 +24,114 @@
 
 #include "usbhid-ups.h"
 #include "mge-hid.h"
-#include "extstate.h" /* for ST_FLAG_STRING */
-#include "main.h"     /* for getval() */
+#include "extstate.h"	/* for ST_FLAG_STRING */
+#include "main.h"	/* for getval() */
 #include "common.h"
 
-#define MGE_HID_VERSION	"MGE HID 1.01"
+#define MGE_HID_VERSION		"MGE HID 1.10"
+#define MGE_VENDORID		0x0463
 
-#define MGE_VENDORID 0x0463
+typedef enum {
+	MGE_DEFAULT = 0,
+	MGE_EVOLUTION = 0x100,		/* MGE Evolution series */
+		MGE_EVOLUTION_650,
+		MGE_EVOLUTION_850,
+		MGE_EVOLUTION_1150,
+		MGE_EVOLUTION_S_1250,
+		MGE_EVOLUTION_1550,
+		MGE_EVOLUTION_S_1750,
+		MGE_EVOLUTION_2000,
+		MGE_EVOLUTION_S_2500,
+		MGE_EVOLUTION_S_3000,
+	MGE_PULSAR_M = 0x200,		/* MGE Pulsar M series */
+		MGE_PULSAR_M_2200,
+		MGE_PULSAR_M_3000,
+		MGE_PULSAR_M_3000_XL
+} models_type_t;
 
+static models_type_t	mge_type = MGE_DEFAULT;
+static char		mge_scratch_buf[16];
 
-/* returns statically allocated string - must not use it again before
-   done with result! */
-static char *mge_battery_voltage_nominal_fun(long value) {
-	static char buf[10];
-	const char *model;
+/* The HID path 'UPS.PowerSummary.ConfigVoltage' only reports
+   'battery.voltage.nominal' for specific UPS series. Ignore
+   the value for other series (default behavior). */
+static char *mge_battery_voltage_nominal_fun(double value)
+{
+	switch (mge_type & 0xFF00)	/* Ignore model byte */
+	{
+	case MGE_EVOLUTION:
+		if (mge_type == MGE_EVOLUTION_650) {
+			value = 12.0;
+		}
+		break;
 
-	model = dstate_getinfo("ups.model");
+	case MGE_PULSAR_M:
+		break;
 
-	/* Work around for Evolution 650 bug(?) */
-	if (!strcmp(model, "Evolution 650"))
-		value = 12;
+	default:
+		return NULL;
+	}
 
-	snprintf(buf, sizeof(buf), "%ld", value);
-	return buf;
+	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.0f", value);
+	return mge_scratch_buf;
 }
 
 static info_lkp_t mge_battery_voltage_nominal[] = {
 	{ 0, NULL, mge_battery_voltage_nominal_fun }
 };
 
-/* returns statically allocated string - must not use it again before
-   done with result! */
-static char *mge_powerfactor_conversion_fun(long value) {
-	static char buf[20];
+/* The HID path 'UPS.PowerSummary.Voltage' only reports
+   'battery.voltage' for specific UPS series. Ignore the
+   value for other series (default behavior). */
+static char *mge_battery_voltage_fun(double value)
+{
+	switch (mge_type & 0xFF00)	/* Ignore model byte */
+	{
+	case MGE_EVOLUTION:
+	case MGE_PULSAR_M:
+		break;
 
-	snprintf(buf, sizeof(buf), "%.2f", (double)value / 100);
-	return buf;
+	default:
+		return NULL;
+	}
+
+	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.1f", value);
+	return mge_scratch_buf;
+}
+
+static info_lkp_t mge_battery_voltage[] = {
+	{ 0, NULL, mge_battery_voltage_fun }
+};
+
+static char *mge_powerfactor_conversion_fun(double value)
+{
+	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.2f", value / 100);
+	return mge_scratch_buf;
 }
 
 static info_lkp_t mge_powerfactor_conversion[] = {
 	{ 0, NULL, mge_powerfactor_conversion_fun }
 };
 
-/* returns statically allocated string - must not use it again before
-   done with result! */
-static char *mge_battery_capacity_fun(long value) {
-	static char buf[10];
-
-	snprintf(buf, sizeof(buf), "%.2f", (double)value / 3600);
-	return buf;
+static char *mge_battery_capacity_fun(double value)
+{
+	snprintf(mge_scratch_buf, sizeof(mge_scratch_buf), "%.2f", value / 3600);
+	return mge_scratch_buf;
 }
 
 static info_lkp_t mge_battery_capacity[] = {
 	{ 0, NULL, mge_battery_capacity_fun }
 };
 
+static info_lkp_t mge_upstype_conversion[] = {
+	{ 1, "offline / line interactive", NULL },
+	{ 2, "online", NULL },
+	{ 3, "online - unitary/parallel", NULL },
+	{ 4, "online - parallel with hot standy", NULL },
+	{ 5, "online - hot standby redundancy", NULL },
+	{ 0, NULL, NULL }
+};
+
 static info_lkp_t mge_sensitivity_info[] = {
 	{ 0, "normal", NULL },
 	{ 1, "high", NULL },
@@ -227,98 +280,120 @@ static usage_tables_t mge_utab[] = {
 /*      Model Name formating entries                               */
 /* --------------------------------------------------------------- */
 
+typedef struct {
+	const char	*iProduct;
+	const char	*iModel;
+	models_type_t	type;		/* Enumerated model type */
+	const char	*name;
+} models_name_t;
+
 static models_name_t mge_model_names [] =
 {
 	/* Ellipse models */
-	{ "ELLIPSE", "300", -1, "ellipse 300" },
-	{ "ELLIPSE", "500", -1, "ellipse 500" },
-	{ "ELLIPSE", "650", -1, "ellipse 650" },
-	{ "ELLIPSE", "800", -1, "ellipse 800" },
-	{ "ELLIPSE", "1200", -1, "ellipse 1200" },
+	{ "ELLIPSE", "300", MGE_DEFAULT, "ellipse 300" },
+	{ "ELLIPSE", "500", MGE_DEFAULT, "ellipse 500" },
+	{ "ELLIPSE", "650", MGE_DEFAULT, "ellipse 650" },
+	{ "ELLIPSE", "800", MGE_DEFAULT, "ellipse 800" },
+	{ "ELLIPSE", "1200", MGE_DEFAULT, "ellipse 1200" },
+
 	/* Ellipse Premium models */
-	{ "ellipse", "PR500", -1, "ellipse premium 500" },
-	{ "ellipse", "PR650", -1, "ellipse premium 650" },
-	{ "ellipse", "PR800", -1, "ellipse premium 800" },
-	{ "ellipse", "PR1200", -1, "ellipse premium 1200" },
+	{ "ellipse", "PR500", MGE_DEFAULT, "ellipse premium 500" },
+	{ "ellipse", "PR650", MGE_DEFAULT, "ellipse premium 650" },
+	{ "ellipse", "PR800", MGE_DEFAULT, "ellipse premium 800" },
+	{ "ellipse", "PR1200", MGE_DEFAULT, "ellipse premium 1200" },
+
 	/* Ellipse "Pro" */
-	{ "ELLIPSE", "600", -1, "Ellipse 600" },
-	{ "ELLIPSE", "750", -1, "Ellipse 750" },
-	{ "ELLIPSE", "1000", -1, "Ellipse 1000" },
-	{ "ELLIPSE", "1500", -1, "Ellipse 1500" },
+	{ "ELLIPSE", "600", MGE_DEFAULT, "Ellipse 600" },
+	{ "ELLIPSE", "750", MGE_DEFAULT, "Ellipse 750" },
+	{ "ELLIPSE", "1000", MGE_DEFAULT, "Ellipse 1000" },
+	{ "ELLIPSE", "1500", MGE_DEFAULT, "Ellipse 1500" },
+
 	/* Ellipse "MAX" */
-	{ "Ellipse MAX", "600", -1, "Ellipse MAX 600" },
-	{ "Ellipse MAX", "850", -1, "Ellipse MAX 850" },
-	{ "Ellipse MAX", "1100", -1, "Ellipse MAX 1100" },
-	{ "Ellipse MAX", "1500", -1, "Ellipse MAX 1500" },
+	{ "Ellipse MAX", "600", MGE_DEFAULT, "Ellipse MAX 600" },
+	{ "Ellipse MAX", "850", MGE_DEFAULT, "Ellipse MAX 850" },
+	{ "Ellipse MAX", "1100", MGE_DEFAULT, "Ellipse MAX 1100" },
+	{ "Ellipse MAX", "1500", MGE_DEFAULT, "Ellipse MAX 1500" },
+
 	/* Protection Center */
-	{ "PROTECTIONCENTER", "420", -1, "Protection Center 420" },
-	{ "PROTECTIONCENTER", "500", -1, "Protection Center 500" },
-	{ "PROTECTIONCENTER", "675", -1, "Protection Center 675" },
+	{ "PROTECTIONCENTER", "420", MGE_DEFAULT, "Protection Center 420" },
+	{ "PROTECTIONCENTER", "500", MGE_DEFAULT, "Protection Center 500" },
+	{ "PROTECTIONCENTER", "675", MGE_DEFAULT, "Protection Center 675" },
+
 	/* Evolution models */
-	{ "Evolution", "500", -1, "Pulsar Evolution 500" },
-	{ "Evolution", "800", -1, "Pulsar Evolution 800" },
-	{ "Evolution", "1100", -1, "Pulsar Evolution 1100" },
-	{ "Evolution", "1500", -1, "Pulsar Evolution 1500" },
-	{ "Evolution", "2200", -1, "Pulsar Evolution 2200" },
-	{ "Evolution", "3000", -1, "Pulsar Evolution 3000" },
-	{ "Evolution", "3000XL", -1, "Pulsar Evolution 3000 XL" },
+	{ "Evolution", "500", MGE_DEFAULT, "Pulsar Evolution 500" },
+	{ "Evolution", "800", MGE_DEFAULT, "Pulsar Evolution 800" },
+	{ "Evolution", "1100", MGE_DEFAULT, "Pulsar Evolution 1100" },
+	{ "Evolution", "1500", MGE_DEFAULT, "Pulsar Evolution 1500" },
+	{ "Evolution", "2200", MGE_DEFAULT, "Pulsar Evolution 2200" },
+	{ "Evolution", "3000", MGE_DEFAULT, "Pulsar Evolution 3000" },
+	{ "Evolution", "3000XL", MGE_DEFAULT, "Pulsar Evolution 3000 XL" },
+
 	/* Newer Evolution models */
-	{ "Evolution", "650", -1, "Evolution 650" },
-	{ "Evolution", "850", -1, "Evolution 850" },
-	{ "Evolution", "1150", -1, "Evolution 1150" },
-	{ "Evolution", "S 1250", -1, "Evolution S 1250" },
-	{ "Evolution", "1550", -1, "Evolution 1550" },
-	{ "Evolution", "S 1750", -1, "Evolution S 1750" },
-	{ "Evolution", "2000", -1, "Evolution 2000" },
-	{ "Evolution", "S 2500", -1, "Evolution S 2500" },
-	{ "Evolution", "S 3000", -1, "Evolution S 3000" },
+	{ "Evolution", "650", MGE_EVOLUTION_650, "Evolution 650" },
+	{ "Evolution", "850", MGE_EVOLUTION_850, "Evolution 850" },
+	{ "Evolution", "1150", MGE_EVOLUTION_1150, "Evolution 1150" },
+	{ "Evolution", "S 1250", MGE_EVOLUTION_S_1250, "Evolution S 1250" },
+	{ "Evolution", "1550", MGE_EVOLUTION_1550, "Evolution 1550" },
+	{ "Evolution", "S 1750", MGE_EVOLUTION_S_1750, "Evolution S 1750" },
+	{ "Evolution", "2000", MGE_EVOLUTION_2000, "Evolution 2000" },
+	{ "Evolution", "S 2500", MGE_EVOLUTION_S_2500, "Evolution S 2500" },
+	{ "Evolution", "S 3000", MGE_EVOLUTION_S_3000, "Evolution S 3000" },
+
 	/* Pulsar M models */
-	{ "PULSAR M", "2200", -1, "Pulsar M 2200" },
-	{ "PULSAR M", "3000", -1, "Pulsar M 3000" },
-	{ "PULSAR M", "3000 XL", -1, "Pulsar M 3000 XL" },
+	{ "PULSAR M", "2200", MGE_PULSAR_M_2200, "Pulsar M 2200" },
+	{ "PULSAR M", "3000", MGE_PULSAR_M_3000, "Pulsar M 3000" },
+	{ "PULSAR M", "3000 XL", MGE_PULSAR_M_3000_XL, "Pulsar M 3000 XL" },
+
 	/* Pulsar models */
-	{ "Pulsar", "700", -1, "Pulsar 700" },
-	{ "Pulsar", "1000", -1, "Pulsar 1000" },
-	{ "Pulsar", "1500", -1, "Pulsar 1500" },
-	{ "Pulsar", "1000 RT2U", -1, "Pulsar 1000 RT2U" },
-	{ "Pulsar", "1500 RT2U", -1, "Pulsar 1500 RT2U" },
+	{ "Pulsar", "700", MGE_DEFAULT, "Pulsar 700" },
+	{ "Pulsar", "1000", MGE_DEFAULT, "Pulsar 1000" },
+	{ "Pulsar", "1500", MGE_DEFAULT, "Pulsar 1500" },
+	{ "Pulsar", "1000 RT2U", MGE_DEFAULT, "Pulsar 1000 RT2U" },
+	{ "Pulsar", "1500 RT2U", MGE_DEFAULT, "Pulsar 1500 RT2U" },
+
 	/* Pulsar MX models */
-	{ "PULSAR", "MX4000", -1, "Pulsar MX 4000 RT" },
-	{ "PULSAR", "MX5000", -1, "Pulsar MX 5000 RT" },
+	{ "PULSAR", "MX4000", MGE_DEFAULT, "Pulsar MX 4000 RT" },
+	{ "PULSAR", "MX5000", MGE_DEFAULT, "Pulsar MX 5000 RT" },
+
 	/* NOVA models */	
-	{ "NOVA AVR", "600", -1, "NOVA 600 AVR" },
-	{ "NOVA AVR", "1100", -1, "NOVA 1100 AVR" },
+	{ "NOVA AVR", "600", MGE_DEFAULT, "NOVA 600 AVR" },
+	{ "NOVA AVR", "1100", MGE_DEFAULT, "NOVA 1100 AVR" },
+
 	/* EXtreme C (EMEA) */
-	{ "EXtreme", "700C", -1, "Pulsar EXtreme 700C" },
-	{ "EXtreme", "1000C", -1, "Pulsar EXtreme 1000C" },
-	{ "EXtreme", "1500C", -1, "Pulsar EXtreme 1500C" },
-	{ "EXtreme", "1500CCLA", -1, "Pulsar EXtreme 1500C CLA" },
-	{ "EXtreme", "2200C", -1, "Pulsar EXtreme 2200C" },
-	{ "EXtreme", "3200C", -1, "Pulsar EXtreme 3200C" },
+	{ "EXtreme", "700C", MGE_DEFAULT, "Pulsar EXtreme 700C" },
+	{ "EXtreme", "1000C", MGE_DEFAULT, "Pulsar EXtreme 1000C" },
+	{ "EXtreme", "1500C", MGE_DEFAULT, "Pulsar EXtreme 1500C" },
+	{ "EXtreme", "1500CCLA", MGE_DEFAULT, "Pulsar EXtreme 1500C CLA" },
+	{ "EXtreme", "2200C", MGE_DEFAULT, "Pulsar EXtreme 2200C" },
+	{ "EXtreme", "3200C", MGE_DEFAULT, "Pulsar EXtreme 3200C" },
+
 	/* EXtreme C (USA, aka "EX RT") */
-	{ "EX", "700RT", -1, "Pulsar EX 700 RT" },
-	{ "EX", "1000RT", -1, "Pulsar EX 1000 RT" },
-	{ "EX", "1500RT", -1, "Pulsar EX 1500 RT" },
-	{ "EX", "2200RT", -1, "Pulsar EX 2200 RT" },
-	{ "EX", "3200RT", -1, "Pulsar EX 3200 RT" },
+	{ "EX", "700RT", MGE_DEFAULT, "Pulsar EX 700 RT" },
+	{ "EX", "1000RT", MGE_DEFAULT, "Pulsar EX 1000 RT" },
+	{ "EX", "1500RT", MGE_DEFAULT, "Pulsar EX 1500 RT" },
+	{ "EX", "2200RT", MGE_DEFAULT, "Pulsar EX 2200 RT" },
+	{ "EX", "3200RT", MGE_DEFAULT, "Pulsar EX 3200 RT" },
+
 	/* Comet EX RT three phased */
-	{ "EX", "5RT31", -1, "EX 5 RT 3:1" },
-	{ "EX", "7RT31", -1, "EX 7 RT 3:1" },
-	{ "EX", "11RT31", -1, "EX 11 RT 3:1" },
+	{ "EX", "5RT31", MGE_DEFAULT, "EX 5 RT 3:1" },
+	{ "EX", "7RT31", MGE_DEFAULT, "EX 7 RT 3:1" },
+	{ "EX", "11RT31", MGE_DEFAULT, "EX 11 RT 3:1" },
+
 	/* Comet EX RT mono phased */
-	{ "EX", "5RT", -1, "EX 5 RT" },
-	{ "EX", "7RT", -1, "EX 7 RT" },
-	{ "EX", "11RT", -1, "EX 11 RT" },
+	{ "EX", "5RT", MGE_DEFAULT, "EX 5 RT" },
+	{ "EX", "7RT", MGE_DEFAULT, "EX 7 RT" },
+	{ "EX", "11RT", MGE_DEFAULT, "EX 11 RT" },
+
 	/* Galaxy 3000 */
-	{ "GALAXY", "3000_10", -1, "Galaxy 3000 10 kVA" },
-	{ "GALAXY", "3000_15", -1, "Galaxy 3000 15 kVA" },
-	{ "GALAXY", "3000_20", -1, "Galaxy 3000 20 kVA" },
-	{ "GALAXY", "3000_30", -1, "Galaxy 3000 30 kVA" },
+	{ "GALAXY", "3000_10", MGE_DEFAULT, "Galaxy 3000 10 kVA" },
+	{ "GALAXY", "3000_15", MGE_DEFAULT, "Galaxy 3000 15 kVA" },
+	{ "GALAXY", "3000_20", MGE_DEFAULT, "Galaxy 3000 20 kVA" },
+	{ "GALAXY", "3000_30", MGE_DEFAULT, "Galaxy 3000 30 kVA" },
 
 	/* FIXME: To be completed (Comet, Galaxy, Esprit, ...) */
 
 	/* end of structure. */
-	{ NULL, NULL, -1, "Generic MGE HID model" }
+	{ NULL, NULL, MGE_DEFAULT, "Generic MGE HID model" }
 };
 
 
@@ -341,7 +416,8 @@ static hid_info_t mge_hid2nut[] =
 	{ "battery.runtime", 0, 0, "UPS.PowerSummary.RunTimeToEmpty", NULL, "%.0f", 0, NULL },
 	{ "battery.temperature", 0, 0, "UPS.BatterySystem.Battery.Temperature", NULL, "%.1f", 0, NULL },
 	{ "battery.type", 0, 0, "UPS.PowerSummary.iDeviceChemistry", NULL, "%s", HU_FLAG_STATIC, stringid_conversion },
-	{ "battery.voltage", 0, 0, "UPS.PowerSummary.Voltage", NULL, "%.1f", 0, NULL },
+	{ "battery.voltage", 0, 0, "UPS.BatterySystem.Voltage", NULL, "%.1f", 0, NULL },
+	{ "battery.voltage", 0, 0, "UPS.PowerSummary.Voltage", NULL, "%s", 0, mge_battery_voltage },
 	{ "battery.voltage.nominal", 0, 0, "UPS.BatterySystem.ConfigVoltage", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "battery.voltage.nominal", 0, 0, "UPS.PowerSummary.ConfigVoltage", NULL, "%s", HU_FLAG_STATIC, mge_battery_voltage_nominal },
 	{ "battery.protection", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.BatterySystem.Battery.DeepDischargeProtection", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
@@ -351,20 +427,29 @@ static hid_info_t mge_hid2nut[] =
 	{ "ups.firmware", 0, 0, "UPS.PowerSummary.iVersion", NULL, "%s", HU_FLAG_STATIC, stringid_conversion },
 	{ "ups.load", 0, 0, "UPS.PowerSummary.PercentLoad", NULL, "%.0f", 0, NULL },
 	{ "ups.load.high", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.Flow.[4].ConfigPercentLoad", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
-	{ "ups.delay.shutdown", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", NULL, "%.0f", 0, NULL},
-	{ "ups.delay.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", NULL, "%.0f", 0, NULL},
-	{ "ups.delay.start", 0, 0, "UPS.PowerSummary.DelayBeforeStartup", NULL, "%.0f", 0, NULL},
+	{ "ups.delay.start", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.PowerSummary.DelayBeforeStartup", NULL, DEFAULT_ONDELAY, HU_FLAG_ABSENT, NULL},
+	{ "ups.delay.shutdown", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.PowerSummary.DelayBeforeShutdown", NULL, DEFAULT_OFFDELAY, HU_FLAG_ABSENT, NULL},
+	{ "ups.timer.start", 0, 0, "UPS.PowerSummary.DelayBeforeStartup", NULL, "%.0f", 0, NULL},
+	{ "ups.timer.shutdown", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", NULL, "%.0f", 0, NULL},
+	{ "ups.timer.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", NULL, "%.0f", 0, NULL},
 	{ "ups.test.result", 0, 0, "UPS.BatterySystem.Battery.Test", NULL, "%s", HU_FLAG_SEMI_STATIC, test_read_info },
 	{ "ups.test.interval", ST_FLAG_RW | ST_FLAG_STRING, 8, "UPS.BatterySystem.Battery.TestPeriod", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "ups.beeper.status", 0 ,0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "%s", HU_FLAG_SEMI_STATIC, beeper_info },
 	{ "ups.temperature", 0, 0, "UPS.PowerSummary.Temperature", NULL, "%.1f", 0, NULL },
 	{ "ups.power", 0, 0, "UPS.PowerConverter.Output.ApparentPower", NULL, "%.0f", 0, NULL },
+	{ "ups.L1.power", 0, 0, "UPS.PowerConverter.Output.Phase.[1].ApparentPower", NULL, "%.0f", 0, NULL },
+	{ "ups.L2.power", 0, 0, "UPS.PowerConverter.Output.Phase.[2].ApparentPower", NULL, "%.0f", 0, NULL },
+	{ "ups.L3.power", 0, 0, "UPS.PowerConverter.Output.Phase.[3].ApparentPower", NULL, "%.0f", 0, NULL },
 	{ "ups.power.nominal", 0, 0, "UPS.Flow.[4].ConfigApparentPower", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "ups.realpower", 0, 0, "UPS.PowerConverter.Output.ActivePower", NULL, "%.0f", 0, NULL },
+	{ "ups.L1.realpower", 0, 0, "UPS.PowerConverter.Output.Phase.[1].ActivePower", NULL, "%.0f", 0, NULL },
+	{ "ups.L2.realpower", 0, 0, "UPS.PowerConverter.Output.Phase.[2].ActivePower", NULL, "%.0f", 0, NULL },
+	{ "ups.L3.realpower", 0, 0, "UPS.PowerConverter.Output.Phase.[3].ActivePower", NULL, "%.0f", 0, NULL },
 	{ "ups.realpower.nominal", 0, 0, "UPS.Flow.[4].ConfigActivePower", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "ups.start.auto", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Input.[1].AutomaticRestart", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
 	{ "ups.start.battery", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Input.[3].StartOnBattery", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
 	{ "ups.start.reboot", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.ForcedReboot", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
+	{ "ups.type", 0, 0, "UPS.PowerConverter.ConverterType", NULL, "%s", HU_FLAG_STATIC, mge_upstype_conversion },
 
 	/* Special case: boolean values that are mapped to ups.status and ups.alarm */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ACPresent", NULL, NULL, HU_FLAG_QUICK_POLL, online_info },
@@ -378,12 +463,10 @@ static hid_info_t mge_hid2nut[] =
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.VoltageOutOfRange", NULL, NULL, 0, vrange_info },
 	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.FrequencyOutOfRange", NULL, NULL, 0, frange_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Good", NULL, NULL, 0, off_info },
-	/* Manual bypass */
-	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[4].PresentStatus.Used", NULL, NULL, 0, bypass_info },
-	/* { "BOOL", 0, 0, "UPS.PowerConverter.Input.[3].PresentStatus.Used", NULL, NULL, 0, onbatt_info }, */
-	/* Automatic bypass */
-	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[2].PresentStatus.Used", NULL, NULL, 0, bypass_info },
 	/* { "BOOL", 0, 0, "UPS.PowerConverter.Input.[1].PresentStatus.Used", NULL, NULL, 0, online_info }, */
+	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[2].PresentStatus.Used", NULL, NULL, 0, bypass_auto_info }, /* Automatic bypass */
+	/* { "BOOL", 0, 0, "UPS.PowerConverter.Input.[3].PresentStatus.Used", NULL, NULL, 0, onbatt_info }, */
+	{ "BOOL", 0, 0, "UPS.PowerConverter.Input.[4].PresentStatus.Used", NULL, NULL, 0, bypass_manual_info }, /* Manual bypass */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.FanFailure", NULL, NULL, 0, fanfail_info },
 	{ "BOOL", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.Present", NULL, NULL, 0, nobattery_info },
 	{ "BOOL", 0, 0, "UPS.BatterySystem.Charger.PresentStatus.InternalFailure", NULL, NULL, 0, chargerfail_info },
@@ -398,11 +481,20 @@ static hid_info_t mge_hid2nut[] =
 
 	/* Input page */
 	{ "input.voltage", 0, 0, "UPS.PowerConverter.Input.[1].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.L1-N.voltage", 0, 0, "UPS.PowerConverter.Input.[1].Phase.[1].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.L2-N.voltage", 0, 0, "UPS.PowerConverter.Input.[1].Phase.[2].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.L3-N.voltage", 0, 0, "UPS.PowerConverter.Input.[1].Phase.[3].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.L1-L2.voltage", 0, 0, "UPS.PowerConverter.Input.[1].Phase.[12].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.L2-L3.voltage", 0, 0, "UPS.PowerConverter.Input.[1].Phase.[23].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.L3-L1.voltage", 0, 0, "UPS.PowerConverter.Input.[1].Phase.[31].Voltage", NULL, "%.1f", 0, NULL },
 	{ "input.voltage.nominal", 0, 0, "UPS.Flow.[1].ConfigVoltage", NULL, "%.0f", HU_FLAG_STATIC, NULL },
-	{ "input.voltage.extended", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.ExtendedVoltageMode", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
+	{ "input.current", 0, 0, "UPS.PowerConverter.Input.[1].Current", NULL, "%.2f", 0, NULL },
+	{ "input.L1.current", 0, 0, "UPS.PowerConverter.Input.[1].Phase.[1].Current", NULL, "%.1f", 0, NULL },
+	{ "input.L2.current", 0, 0, "UPS.PowerConverter.Input.[1].Phase.[2].Current", NULL, "%.1f", 0, NULL },
+	{ "input.L3.current", 0, 0, "UPS.PowerConverter.Input.[1].Phase.[3].Current", NULL, "%.1f", 0, NULL },
+	{ "input.current.nominal", 0, 0, "UPS.Flow.[1].ConfigCurrent", NULL, "%.2f", HU_FLAG_STATIC, NULL },
 	{ "input.frequency", 0, 0, "UPS.PowerConverter.Input.[1].Frequency", NULL, "%.1f", 0, NULL },
 	{ "input.frequency.nominal", 0, 0, "UPS.Flow.[1].ConfigFrequency", NULL, "%.0f", HU_FLAG_STATIC, NULL },
-	{ "input.frequency.extended", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.ExtendedFrequencyMode", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
 	/* same as "input.transfer.boost.low" */
 	{ "input.transfer.low", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.LowVoltageTransfer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "input.transfer.boost.low", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.LowVoltageBoostTransfer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
@@ -412,22 +504,50 @@ static hid_info_t mge_hid2nut[] =
 	{ "input.transfer.high", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.HighVoltageTransfer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "input.transfer.trim.high", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.HighVoltageBuckTransfer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "input.sensitivity", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.PowerConverter.Output.SensitivityMode", NULL, "%s", HU_FLAG_SEMI_STATIC, mge_sensitivity_info },
+	{ "input.voltage.extended", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.ExtendedVoltageMode", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
+	{ "input.frequency.extended", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.ExtendedFrequencyMode", NULL, "%s", HU_FLAG_SEMI_STATIC, yes_no_info },
+
+	/* Bypass page */
+	{ "input.bypass.voltage", 0, 0, "UPS.PowerConverter.Input.[2].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.L1-N.voltage", 0, 0, "UPS.PowerConverter.Input.[2].Phase.[1].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.L2-N.voltage", 0, 0, "UPS.PowerConverter.Input.[2].Phase.[2].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.L3-N.voltage", 0, 0, "UPS.PowerConverter.Input.[2].Phase.[3].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.L1-L2.voltage", 0, 0, "UPS.PowerConverter.Input.[2].Phase.[12].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.L2-L3.voltage", 0, 0, "UPS.PowerConverter.Input.[2].Phase.[23].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.L3-L1.voltage", 0, 0, "UPS.PowerConverter.Input.[2].Phase.[31].Voltage", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.voltage.nominal", 0, 0, "UPS.Flow.[2].ConfigVoltage", NULL, "%.0f", HU_FLAG_STATIC, NULL },
+	{ "input.bypass.current", 0, 0, "UPS.PowerConverter.Input.[2].Current", NULL, "%.2f", 0, NULL },
+	{ "input.bypass.L1.current", 0, 0, "UPS.PowerConverter.Input.[2].Phase.[1].Current", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.L2.current", 0, 0, "UPS.PowerConverter.Input.[2].Phase.[2].Current", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.L3.current", 0, 0, "UPS.PowerConverter.Input.[2].Phase.[3].Current", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.current.nominal", 0, 0, "UPS.Flow.[2].ConfigCurrent", NULL, "%.2f", HU_FLAG_STATIC, NULL },
+	{ "input.bypass.frequency", 0, 0, "UPS.PowerConverter.Input.[2].Frequency", NULL, "%.1f", 0, NULL },
+	{ "input.bypass.frequency.nominal", 0, 0, "UPS.Flow.[2].ConfigFrequency", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 
 	/* Output page */
 	{ "output.voltage", 0, 0, "UPS.PowerConverter.Output.Voltage", NULL, "%.1f", 0, NULL },
+	{ "output.L1-N.voltage", 0, 0, "UPS.PowerConverter.Output.Phase.[1].Voltage", NULL, "%.1f", 0, NULL },
+	{ "output.L2-N.voltage", 0, 0, "UPS.PowerConverter.Output.Phase.[2].Voltage", NULL, "%.1f", 0, NULL },
+	{ "output.L3-N.voltage", 0, 0, "UPS.PowerConverter.Output.Phase.[3].Voltage", NULL, "%.1f", 0, NULL },
+	{ "output.L1-L2.voltage", 0, 0, "UPS.PowerConverter.Output.Phase.[12].Voltage", NULL, "%.1f", 0, NULL },
+	{ "output.L2-L3.voltage", 0, 0, "UPS.PowerConverter.Output.Phase.[23].Voltage", NULL, "%.1f", 0, NULL },
+	{ "output.L3-L1.voltage", 0, 0, "UPS.PowerConverter.Output.Phase.[31].Voltage", NULL, "%.1f", 0, NULL },
 	{ "output.voltage.nominal", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.Flow.[4].ConfigVoltage", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "output.current", 0, 0, "UPS.PowerConverter.Output.Current", NULL, "%.2f", 0, NULL },
-	{ "output.powerfactor", 0, 0, "UPS.PowerConverter.Output.PowerFactor", NULL, "%s", 0, mge_powerfactor_conversion },
+	{ "output.L1.current", 0, 0, "UPS.PowerConverter.Output.Phase.[1].Current", NULL, "%.1f", 0, NULL },
+	{ "output.L2.current", 0, 0, "UPS.PowerConverter.Output.Phase.[2].Current", NULL, "%.1f", 0, NULL },
+	{ "output.L3.current", 0, 0, "UPS.PowerConverter.Output.Phase.[3].Current", NULL, "%.1f", 0, NULL },
+	{ "output.current.nominal", 0, 0, "UPS.Flow.[4].ConfigCurrent", NULL, "%.2f", 0, NULL },
 	{ "output.frequency", 0, 0, "UPS.PowerConverter.Output.Frequency", NULL, "%.1f", 0, NULL },
 	{ "output.frequency.nominal", 0, 0, "UPS.Flow.[4].ConfigFrequency", NULL, "%.0f", HU_FLAG_STATIC, NULL },
+	{ "output.powerfactor", 0, 0, "UPS.PowerConverter.Output.PowerFactor", NULL, "%s", 0, mge_powerfactor_conversion },
 
 	/* Outlet page (using MGE UPS SYSTEMS - PowerShare technology) */
 	{ "outlet.0.id", 0, 0, "UPS.OutletSystem.Outlet.[1].OutletID", NULL, "%.0f", HU_FLAG_STATIC, NULL },
-	{ "outlet.0.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[1].OutletID", NULL, "Main Outlet",
-		HU_FLAG_ABSENT | HU_FLAG_STATIC, NULL },
+	{ "outlet.0.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[1].OutletID", NULL, "Main Outlet", HU_FLAG_ABSENT, NULL },
 	{ "outlet.0.switchable", 0, 0, "UPS.OutletSystem.Outlet.[1].PresentStatus.Switchable", NULL, "%s", HU_FLAG_STATIC, yes_no_info },
 	{ "outlet.1.id", 0, 0, "UPS.OutletSystem.Outlet.[2].OutletID", NULL, "%.0f", HU_FLAG_STATIC, NULL },
-	{ "outlet.1.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[2].OutletID", NULL, "PowerShare Outlet 1", HU_FLAG_ABSENT | HU_FLAG_STATIC, NULL },
+	{ "outlet.1.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[2].OutletID", NULL, "PowerShare Outlet 1", HU_FLAG_ABSENT, NULL },
 	{ "outlet.1.switchable", 0, 0, "UPS.OutletSystem.Outlet.[2].PresentStatus.Switchable", NULL, "%s", HU_FLAG_STATIC, yes_no_info },
 	{ "outlet.1.status", 0, 0, "UPS.OutletSystem.Outlet.[2].PresentStatus.SwitchOn/Off", NULL, "%s", 0, on_off_info },
 	/* For low end models, with 1 non backup'ed outlet */
@@ -436,7 +556,7 @@ static hid_info_t mge_hid2nut[] =
 	{ "outlet.1.delay.shutdown", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.OutletSystem.Outlet.[2].ShutdownTimer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "outlet.1.delay.start", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.OutletSystem.Outlet.[2].StartupTimer", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "outlet.2.id", 0, 0, "UPS.OutletSystem.Outlet.[3].OutletID", NULL, "%.0f", HU_FLAG_STATIC, NULL },
-	{ "outlet.2.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[3].OutletID", NULL, "PowerShare Outlet 2", HU_FLAG_ABSENT | HU_FLAG_STATIC, NULL },
+	{ "outlet.2.desc", ST_FLAG_RW | ST_FLAG_STRING, 20, "UPS.OutletSystem.Outlet.[3].OutletID", NULL, "PowerShare Outlet 2", HU_FLAG_ABSENT, NULL },
 	{ "outlet.2.switchable", 0, 0, "UPS.OutletSystem.Outlet.[3].PresentStatus.Switchable", NULL, "%s", HU_FLAG_STATIC, yes_no_info },
 	{ "outlet.2.status", 0, 0, "UPS.OutletSystem.Outlet.[3].PresentStatus.SwitchOn/Off", NULL, "%s", 0, on_off_info },
 	{ "outlet.2.autoswitch.charge.low", ST_FLAG_RW | ST_FLAG_STRING, 3, "UPS.OutletSystem.Outlet.[3].RemainingCapacityLimit", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
@@ -450,12 +570,10 @@ static hid_info_t mge_hid2nut[] =
 	{ "test.battery.start.quick", 0, 0, "UPS.BatterySystem.Battery.Test", NULL, "1", HU_TYPE_CMD, NULL },
 	{ "test.battery.start.deep", 0, 0, "UPS.BatterySystem.Battery.Test", NULL, "2", HU_TYPE_CMD, NULL },
 	{ "test.battery.stop", 0, 0, "UPS.BatterySystem.Battery.Test", NULL, "3", HU_TYPE_CMD, NULL },
-	{ "load.off", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", NULL, "0", HU_TYPE_CMD, NULL },
-	{ "load.on", 0, 0, "UPS.PowerSummary.DelayBeforeStartup", NULL, "0", HU_TYPE_CMD, NULL },
-	{ "shutdown.stayoff", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", NULL, "20", HU_TYPE_CMD, NULL },
-	{ "shutdown.return", 0, 0, "UPS.PowerSummary.DelayBeforeStartup", NULL, "30", HU_TYPE_CMD, NULL },
-	{ "shutdown.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", NULL, "10", HU_TYPE_CMD, NULL},
+	{ "load.off.delay", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", NULL, DEFAULT_OFFDELAY, HU_TYPE_CMD, NULL },
+	{ "load.on.delay", 0, 0, "UPS.PowerSummary.DelayBeforeStartup", NULL, DEFAULT_ONDELAY, HU_TYPE_CMD, NULL },
 	{ "shutdown.stop", 0, 0, "UPS.PowerSummary.DelayBeforeShutdown", NULL, "-1", HU_TYPE_CMD, NULL },
+	{ "shutdown.reboot", 0, 0, "UPS.PowerSummary.DelayBeforeReboot", NULL, "10", HU_TYPE_CMD, NULL},
 	{ "beeper.off", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "1", HU_TYPE_CMD, NULL },
 	{ "beeper.on", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "2", HU_TYPE_CMD, NULL },
 	{ "beeper.mute", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "3", HU_TYPE_CMD, NULL },
@@ -473,34 +591,37 @@ static hid_info_t mge_hid2nut[] =
 };
 
 /* All the logic for finely formatting the MGE model name */
-static char *get_model_name(const char *iProduct, char *iModel)
+static const char *get_model_name(const char *iProduct, const char *iModel)
 {
-	models_name_t *model = NULL;
+	models_name_t	*model = NULL;
 
 	upsdebugx(2, "get_model_name(%s, %s)\n", iProduct, iModel);
 
 	/* Search for formatting rules */
-	for (model = mge_model_names; model->iProduct; model++)
-	{
-		upsdebugx(2, "comparing with: %s", model->finalname);
+	for (model = mge_model_names; model->iProduct; model++) {
+		upsdebugx(2, "comparing with: %s", model->name);
 
-		if (strncmp(iProduct, model->iProduct, strlen(model->iProduct)))
+		if (strncmp(iProduct, model->iProduct, strlen(model->iProduct))) {
 			continue;
+		}
 
-		if (strncmp(iModel, model->iModel, strlen(model->iModel)))
+		if (strncmp(iModel, model->iModel, strlen(model->iModel))) {
 			continue;
+		}
 
-		upsdebugx(2, "Found %s\n", model->finalname);
+		upsdebugx(2, "Found %s\n", model->name);
 		break;
 	}
 
-	return model->finalname;
+	mge_type = model->type;		/* enumerated model type */
+
+	return model->name;
 }
 
 static char *mge_format_model(HIDDevice_t *hd) {
-	char *product;
-	char model[64];
-	double value;
+	char	*product;
+	char	model[64];
+	double	value;
 
 	/* Get iProduct and iModel strings */
 	product = hd->Product ? hd->Product : "unknown";
@@ -539,9 +660,9 @@ static int mge_claim(HIDDevice_t *hd) {
 	}
 	switch (hd->ProductID)
 	{
-	case  0x0001:
-	case  0xffff:
-		return 1;  /* accept known UPSs */
+	case 0x0001:
+	case 0xffff:
+		return 1;	/* accept known UPSs */
 
 	default:
 		if (getval("productid")) {
