@@ -23,7 +23,6 @@
  *
  */
 
-#include <sys/ioctl.h>
 #include <string.h>
 
 #include "config.h"
@@ -72,10 +71,10 @@ static void align_request(hid_packet_t *sd)
 /*                      Global structures                          */
 /* --------------------------------------------------------------- */
 
-hid_desc_data_u    	hid_descriptor;
-device_desc_data_u 	device_descriptor;
+hid_desc_data_t    	hid_descriptor;
+device_desc_data_t 	device_descriptor;
 static long             hValue;
-static HIDDesc  	*pDesc = NULL; /* parsed Report Descriptor */
+static HIDDesc_t  	*pDesc = NULL; /* parsed Report Descriptor */
 u_char 			raw_buf[4096];
 
 /* --------------------------------------------------------------- */
@@ -92,7 +91,7 @@ static char *hu_find_infoval(info_lkp_t *hid2info, long value);
 
 void upsdrv_initinfo (void)
 {
-	mge_info_item *item;
+	mge_info_item_t *item;
 	
 	upsdebugx(2, "entering initinfo()\n");
 	
@@ -172,7 +171,7 @@ void upsdrv_initinfo (void)
 
 void upsdrv_updateinfo (void)
 {
-	mge_info_item *item;
+	mge_info_item_t *item;
 	char *nutvalue;
 	
 	upsdebugx(2, "entering upsdrv_updateinfo()");
@@ -181,7 +180,7 @@ void upsdrv_updateinfo (void)
 		if (shut_ups_start () != 0) {
 			upsdebugx(2, "No communication with UPS, retrying");
 			dstate_datastale();
-			sleep (10);
+			return;
 		} else {
 			upsdebugx(2, "Communication with UPS established");
 		}
@@ -235,12 +234,12 @@ void upsdrv_shutdown (void)
 
 	if (sdtype == SD_RETURN) {
 		/* set DelayBeforeStartup */
-		sprintf(val, "%d", ondelay);
+		snprintf(val, sizeof(val), "%d", ondelay);
 		hid_set_value("ups.delay.start", val);
 	}
 
 	/* set DelayBeforeShutdown */
-	sprintf(val, "%d", offdelay);
+	snprintf(val, sizeof(val), "%d", offdelay);
 	hid_set_value("ups.delay.shutdown", val);
 }
 
@@ -260,19 +259,19 @@ void upsdrv_makevartable (void)
 	
 	upsdebugx (2, "entering upsdrv_makevartable()");
   
-	sprintf(msg, "Set low battery level, in %% (default=%d).",
+	snprintf(msg, sizeof(msg), "Set low battery level, in %% (default=%d).",
 		DEFAULT_LOWBATT);
 	addvar (VAR_VALUE, "lowbatt", msg);
 	
-	sprintf(msg, "Set shutdown delay, in seconds (default=%d).",
+	snprintf(msg, sizeof(msg), "Set shutdown delay, in seconds (default=%d).",
 		DEFAULT_OFFDELAY);
 	addvar (VAR_VALUE, "offdelay", msg);
 
-	sprintf(msg, "Set startup delay, in ten seconds units (default=%d).",
+	snprintf(msg, sizeof(msg), "Set startup delay, in ten seconds units (default=%d).",
 		DEFAULT_ONDELAY);
 	addvar (VAR_VALUE, "ondelay", msg);
 	
-	sprintf(msg, "Set notification type, 1 = no, 2 = light, 3 = yes (default=%d).",
+	snprintf(msg, sizeof(msg), "Set notification type, 1 = no, 2 = light, 3 = yes (default=%d).",
 		DEFAULT_NOTIFICATION);
 	addvar (VAR_VALUE, "notification", msg);
 }
@@ -314,13 +313,13 @@ void upsdrv_initups (void)
 
 	/* initialise communication */
 	if (shut_ups_start () != 0)
-		fatalx("No communication with UPS");
+		fatalx(EXIT_FAILURE, "No communication with UPS");
 	else
 		upsdebugx(2, "Communication with UPS established");
 
 	/* initialise HID communication */
 	if(hid_init_device() != 0)
-		fatalx("Can't initialise HID device");
+		fatalx(EXIT_FAILURE, "Can't initialise HID device");
 }
 
 /* --------------------------------------------------------------- */
@@ -470,7 +469,7 @@ int shut_identify_ups ()
 				/* Try with "UPS.Flow.[4].ConfigApparentPower" */
 				if(hid_get_value("UPS.Flow.[4].ConfigApparentPower") != 0 )
 				{
-					sprintf(&string[0], "%i", (int)hValue);
+					snprintf(string, sizeof(string), "%i", (int)hValue);
 					finalname = get_model_name(model, string);
 				}
 				else
@@ -729,10 +728,10 @@ int shut_token_send(u_char token)
 	return serial_send (Buf, 1);
 }
 
-int shut_packet_send (hid_data_u *hdata, int datalen, u_char token)
+int shut_packet_send (hid_data_t *hdata, int datalen, u_char token)
 {
 	shut_packet_t SHUTRequest;
-	shut_data_u   sdata;
+	shut_data_t   sdata;
 	short Retry=1;
 	short Size;
 	int i;
@@ -793,7 +792,7 @@ int shut_packet_recv (u_char *Buf, int datalen)
 	u_short  Pos=0;
 	u_char   Retry=0;
 	int recv;
-	shut_data_u   sdata;
+	shut_data_t   sdata;
 
 	upsdebugx (4, "entering shut_packet_recv (%i)", datalen);
 	
@@ -883,7 +882,7 @@ int shut_packet_recv (u_char *Buf, int datalen)
 int shut_get_descriptor(int desctype, u_char *pkt, int reportlen)
 {
 	hid_packet_t HIDRequest;
-	hid_data_u   data;
+	hid_data_t   data;
 	int retcode;
 	
 	upsdebugx (2, "entering shut_get_descriptor(n %02x, %i)",
@@ -928,7 +927,7 @@ int shut_get_descriptor(int desctype, u_char *pkt, int reportlen)
 int shut_get_string(int strindex, char *string, int stringlen)
 {
 	hid_packet_t HIDRequest;
-	hid_data_u   data;
+	hid_data_t   data;
 	int retcode;
 	u_char buf[MAX_STRING];
 	
@@ -975,7 +974,7 @@ int shut_get_string(int strindex, char *string, int stringlen)
 int shut_get_report(int id, u_char *pkt, int reportlen)
 {
 	hid_packet_t HIDRequest;
-	hid_data_u   data;
+	hid_data_t   data;
 	int retcode;
 	
 	upsdebugx (2, "entering shut_get_report(id: %02x, len: %02x)", id, reportlen);
@@ -1019,7 +1018,7 @@ int shut_get_report(int id, u_char *pkt, int reportlen)
 int shut_set_report(int id, u_char *pkt, int reportlen)
 {
 	hid_packet_t HIDRequest;
-	hid_data_u   data;
+	hid_data_t   data;
 	int retcode;
 	
 	upsdebugx (2, "entering shut_set_report(id: %02x, len: %02x)", id, reportlen);
@@ -1118,23 +1117,23 @@ int hid_init_device()
 				Free_ReportDesc(pDesc);
 				pDesc = Parse_ReportDesc(raw_buf, retcode);
 				if (!pDesc) {
-					fatalx("Failed to parse report descriptor: %s", strerror(errno));
+					fatalx(EXIT_FAILURE, "Failed to parse report descriptor: %s", strerror(errno));
 				}
 			}
 			else
-				fatalx("Unable to get Report Descriptor");
+				fatalx(EXIT_FAILURE, "Unable to get Report Descriptor");
 		}
 		else
-			fatalx("Unable to get Device Descriptor");
+			fatalx(EXIT_FAILURE, "Unable to get Device Descriptor");
 	}
 	else
-		fatalx("Unable to get HID Descriptor");
+		fatalx(EXIT_FAILURE, "Unable to get HID Descriptor");
 
 	return 0;
 }
 
 /* translate HID string path to numeric path and return path depth */
-ushort lookup_path(const char *HIDpath, HIDData *data)
+ushort lookup_path(const char *HIDpath, HIDData_t *data)
 {
 	ushort i = 0, cond = 1;
 	int cur_usage;
@@ -1201,7 +1200,7 @@ int hid_lookup_usage(char *name)
 int hid_get_value(const char *item_path)
 {
 	int i, retcode;
-   HIDData hData;
+   HIDData_t hData;
 	
 	upsdebugx(3, "entering hid_get_value(%s)", item_path);
 	
@@ -1278,18 +1277,15 @@ void make_string(u_char *buf, int datalen, char *string)
  */
 void setline (int set)
 {
-	int dtr = TIOCM_DTR;
-	int rts = TIOCM_RTS;
-	
 	upsdebugx(3, "entering setline(%i)\n", set);
 
 	if (set == 1) {
-		ioctl (upsfd, TIOCMBIC, &dtr);
-		ioctl (upsfd, TIOCMBIS, &rts);
+		ser_set_dtr(upsfd, 0);
+		ser_set_rts(upsfd, 1);
 	}
 	else {
-		ioctl (upsfd, TIOCMBIS, &dtr);
-		ioctl (upsfd, TIOCMBIC, &rts);
+		ser_set_dtr(upsfd, 1);
+		ser_set_rts(upsfd, 0);
 	}
 }
 
@@ -1335,8 +1331,8 @@ char *get_model_name(char *iProduct, char *iModel)
 int hid_set_value(const char *varname, const char *val)
 {
 	int retcode, i, replen;
-	mge_info_item *shut_info_p;
-   HIDData hData;
+	mge_info_item_t *shut_info_p;
+   HIDData_t hData;
 		
 	upsdebugx(2, "============== entering hid_set_value(%s, %s) ==============", varname, val);
 	
@@ -1421,15 +1417,15 @@ int hid_set_value(const char *varname, const char *val)
 }
 
 /* find info element definition in my info array. */
-mge_info_item *shut_find_info(const char *varname)
+mge_info_item_t *shut_find_info(const char *varname)
 {
-	mge_info_item *shut_info_p;
+	mge_info_item_t *shut_info_p;
 
 	for (shut_info_p = &mge_info[0]; shut_info_p->type != NULL; shut_info_p++)
 		if (!strcasecmp(shut_info_p->type, varname))
 			return shut_info_p;
 		
-	fatalx("shut_find_info: unknown info type: %s", varname);
+	fatalx(EXIT_FAILURE, "shut_find_info: unknown info type: %s", varname);
 	return NULL;
 }
 
