@@ -30,15 +30,15 @@
 #define DRV_VERSION "0.50"
 
 #include <stdio.h>
-#include <time.h>
 #include <math.h>
-#include <sys/ioctl.h>
+
 #include "main.h"
 #include "serial.h"
+#include "timehead.h"
 
 #define UPSDELAY 500 /* 0.5 ms delay */
 
-typedef int bool;
+typedef int bool_t;
 
 #define false 0
 #define true 1
@@ -86,12 +86,12 @@ static int ihour,imin, isec;
 // internal variables
 // package handshake ariables
 /* int ContadorEstouro; */
-static bool detected;
-static bool SourceFail, Out110, RedeAnterior, OcorrenciaDeFalha;
-static bool RetornoDaRede, SuperAquecimento, SuperAquecimentoAnterior;
-static bool OverCharge, OldOverCharge, CriticBatt, OldCritBatt;
-static bool Flag_inversor, BypassOn, InputOn, OutputOn;
-static bool LowBatt, oldInversorOn;
+static bool_t detected;
+static bool_t SourceFail, Out110, RedeAnterior, OcorrenciaDeFalha;
+static bool_t RetornoDaRede, SuperAquecimento, SuperAquecimentoAnterior;
+static bool_t OverCharge, OldOverCharge, CriticBatt, OldCritBatt;
+static bool_t Flag_inversor, BypassOn, InputOn, OutputOn;
+static bool_t LowBatt, oldInversorOn;
 /* data vetor from received and configuration data package - not used yet
 unsigned char Dados[ 161 ]; */
 /* identification group */
@@ -573,9 +573,7 @@ static void getbaseinfo(void)
 
 	if( (!detected) )
 	  {
-	    printf( NO_RHINO );
-	    upsdrv_cleanup();
-	    exit(0);
+	    fatalx(EXIT_FAILURE,  NO_RHINO );
 	  }
 
 	switch( RhinoModel )
@@ -797,15 +795,12 @@ void upsdrv_banner(void)
 
 void upsdrv_initups(void)
 {
-	int     dtr_bit = TIOCM_DTR;
-        int     rts_bit = TIOCM_RTS;
-
 	upsfd = ser_open(device_path);
 	ser_set_speed(upsfd, device_path, B19200);
 
 	/* dtr and rts setting */
-	ioctl(upsfd, TIOCMBIS, &dtr_bit);
-	ioctl(upsfd, TIOCMBIC, &rts_bit);
+	ser_set_dtr(upsfd, 1);
+	ser_set_rts(upsfd, 0);
 	
 }
 
