@@ -3,6 +3,7 @@
  *  Copyright (C)  
  *	2003 - 2005 Arnaud Quette <arnaud.quette@free.fr>
  *	2005 - 2006 Peter Selinger <selinger@users.sourceforge.net>
+ *	2008        Arjen de Korte <adkorte-guest@alioth.debian.org>
  *
  *  Sponsored by MGE UPS SYSTEMS <http://www.mgeups.com>
  *
@@ -33,9 +34,16 @@
 
 #define TRIPPLITE_VENDORID 0x09ae 
 
+/* For some devices, the reported battery voltage is off by
+ * factor of 10 (due to an error in the report descriptor),
+ * so we need to apply a scale factor to it to get the actual
+ * battery voltage. By default, the factor is 1 (no scaling).
+ */
+static double	battery_scale = 1.0;
+
 /* returns statically allocated string - must not use it again before
    done with result! */
-static char *tripplite_chemistry_fun(long value)
+static char *tripplite_chemistry_fun(double value)
 {
 	static char	buf[20];
 	const char	*model;
@@ -47,13 +55,33 @@ static char *tripplite_chemistry_fun(long value)
 		return "unknown";
 	}
 
-        return HIDGetIndexString(udev, value, buf, sizeof(buf));
+	/* Workaround for OMNI1000LCD firmware bug */
+	if (!strcmp(model, "2005")) {
+		return "unknown";
+	}
+
+	return HIDGetIndexString(udev, (int)value, buf, sizeof(buf));
 }
 
 static info_lkp_t tripplite_chemistry[] = {
 	{ 0, NULL, tripplite_chemistry_fun }
 };
 
+/* returns statically allocated string - must not use it again before
+   done with result! */
+static char *tripplite_battvolt_fun(double value)
+{
+	static char	buf[8];
+
+	snprintf(buf, sizeof(buf), "%.1f", battery_scale * value);
+
+	return buf;
+}
+
+static info_lkp_t tripplite_battvolt[] = {
+	{ 0, NULL, tripplite_battvolt_fun }
+};
+
 /* --------------------------------------------------------------- */
 /*	Vendor-specific usage table */
 /* --------------------------------------------------------------- */
@@ -62,15 +90,20 @@ static info_lkp_t tripplite_chemistry[] = {
 static usage_lkp_t tripplite_usage_lkp[] = {
 	/* currently unknown: 
 	   ffff0010, 00ff0001, ffff007d, ffff00c0, ffff00c1, ffff00c2,
-	   ffff00c3, ffff00c4, ffff00c5, ffff00d2, ffff0091, ffff0092,
-	   ffff00c7 */
+	   ffff00c3, ffff00c4, ffff00c5, ffff00d2, ffff0091, ffff00c7 */
+
+	{ "TLLowVoltageTransferMax",	0xffff0057 },
+	{ "TLLowVoltageTransferMin",	0xffff0058 },
+	{ "TLHighVoltageTransferMax",	0xffff0059 },
+	{ "TLHighVoltageTransferMin",	0xffff005a },
+	{ "TLWatchdog",			0xffff0092 },
 
 	/* it looks like Tripp Lite confused pages 0x84 and 0x85 for the
 	   following 4 items, on some OMNI1000LCD devices. */
-	{ "TLCharging",				0x00840044 },  /* conflicts with HID spec! */
-	{ "TLDischarging",			0x00840045 },  /* conflicts with HID spec! */
-	{ "TLNeedReplacement",			0x0084004b },
-	{ "TLACPresent",			0x008400d0 },
+	{ "TLCharging",			0x00840044 },  /* conflicts with HID spec! */
+	{ "TLDischarging",		0x00840045 },  /* conflicts with HID spec! */
+	{ "TLNeedReplacement",		0x0084004b },
+	{ "TLACPresent",		0x008400d0 },
 	{ NULL, 0 }
 };
 
@@ -89,31 +122,16 @@ static hid_info_t tripplite_hid2nut[] = {
 
 #ifdef USBHID_UPS_TRIPPLITE_DEBUG
 
-	/* duplicated variables, also: UPS.PowerSummary.PresentStatus.* or UPS.PowerConverter.PresentStatus.* */
-	{ "UPS.BatterySystem.Battery.PresentStatus.Charging", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.Charging", NULL, "%.0f", 0, NULL }, 
-	{ "UPS.BatterySystem.Battery.PresentStatus.Discharging", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.Discharging", NULL, "%.0f", 0, NULL }, 
-	{ "UPS.BatterySystem.Battery.PresentStatus.NeedReplacement", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.NeedReplacement", NULL, "%.0f", 0, NULL }, 
-	{ "UPS.PowerSummary.PresentStatus.InternalFailure", 0, 0, "UPS.PowerSummary.PresentStatus.InternalFailure", NULL, "%.0f", 0, NULL }, 
-	{ "UPS.BatterySystem.Battery.PresentStatus.FullyCharged", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.FullyCharged", NULL, "%.0f", 0, NULL },
-	{ "UPS.BatterySystem.Battery.PresentStatus.BelowRemainingCapacityLimit", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.BelowRemainingCapacityLimit", NULL, "%.0f", 0, NULL },
-
-	/* uninteresting variables: CapacityMode is usually 2=percent; 
-      FullChargeCapacity is usually 100 */
-	{ "UPS.PowerSummary.CapacityMode", 0, 0, "UPS.PowerSummary.CapacityMode", NULL, "%.0f", 0, NULL },
-	{ "UPS.PowerSummary.FullChargeCapacity", 0, 0, "UPS.PowerSummary.FullChargeCapacity", NULL, "%.0f", 0, NULL },
-
-	/* more unmapped variables - meaning unknown */
-	{ "UPS.PowerConverter.PresentStatus.Used", 0, 0, "UPS.PowerConverter.PresentStatus.Used", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0010.00ff0001.ffff007d", 0, 0, "UPS.ffff0010.00ff0001.ffff007d", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.00ff0001.ffff00c0", 0, 0, "UPS.ffff0015.00ff0001.ffff00c0", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.00ff0001.ffff00c1", 0, 0, "UPS.ffff0015.00ff0001.ffff00c1", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.00ff0001.ffff00c2", 0, 0, "UPS.ffff0015.00ff0001.ffff00c2", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.00ff0001.ffff00c3", 0, 0, "UPS.ffff0015.00ff0001.ffff00c3", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.00ff0001.ffff00c4", 0, 0, "UPS.ffff0015.00ff0001.ffff00c4", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.00ff0001.ffff00c5", 0, 0, "UPS.ffff0015.00ff0001.ffff00c5", NULL, "%.0f", 0, NULL },
-	{ "UPS.ffff0015.00ff0001.ffff00d2", 0, 0, "UPS.ffff0015.00ff0001.ffff00d2", NULL, "%.0f", 0, NULL },
+	/* unmapped variables - meaning unknown */
+	{ "UPS.ffff0010.[1].ffff007d", 0, 0, "UPS.ffff0010.[1].ffff007d", NULL, "%.0f", 0, NULL },
+	{ "UPS.ffff0015.[1].ffff00c0", 0, 0, "UPS.ffff0015.[1].ffff00c0", NULL, "%.0f", 0, NULL },
+	{ "UPS.ffff0015.[1].ffff00c1", 0, 0, "UPS.ffff0015.[1].ffff00c1", NULL, "%.0f", 0, NULL },
+	{ "UPS.ffff0015.[1].ffff00c2", 0, 0, "UPS.ffff0015.[1].ffff00c2", NULL, "%.0f", 0, NULL },
+	{ "UPS.ffff0015.[1].ffff00c3", 0, 0, "UPS.ffff0015.[1].ffff00c3", NULL, "%.0f", 0, NULL },
+	{ "UPS.ffff0015.[1].ffff00c4", 0, 0, "UPS.ffff0015.[1].ffff00c4", NULL, "%.0f", 0, NULL },
+	{ "UPS.ffff0015.[1].ffff00c5", 0, 0, "UPS.ffff0015.[1].ffff00c5", NULL, "%.0f", 0, NULL },
+	{ "UPS.ffff0015.[1].ffff00d2", 0, 0, "UPS.ffff0015.[1].ffff00d2", NULL, "%.0f", 0, NULL },
 	{ "UPS.OutletSystem.Outlet.ffff0091", 0, 0, "UPS.OutletSystem.Outlet.ffff0091", NULL, "%.0f", 0, NULL },
-	{ "UPS.OutletSystem.Outlet.ffff0092", 0, 0, "UPS.OutletSystem.Outlet.ffff0092", NULL, "%.0f", 0, NULL },
 	{ "UPS.OutletSystem.Outlet.ffff00c7", 0, 0, "UPS.OutletSystem.Outlet.ffff00c7", NULL, "%.0f", 0, NULL },
 
 #endif /* USBHID_UPS_TRIPPLITE_DEBUG */
@@ -125,81 +143,98 @@ static hid_info_t tripplite_hid2nut[] = {
 	/* Battery page */
 	{ "battery.charge", 0, 0, "UPS.PowerSummary.RemainingCapacity", NULL, "%.0f", 0, NULL },
 	{ "battery.charge", 0, 0, "UPS.BatterySystem.Battery.RemainingCapacity", NULL, "%.0f", 0, NULL },
-	{ "battery.charge.low", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.PowerSummary.RemainingCapacityLimit", NULL, "%.0f", 0, NULL },
+	{ "battery.charge.low", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerSummary.RemainingCapacityLimit", NULL, "%.0f", HU_FLAG_SEMI_STATIC, NULL },
 	{ "battery.charge.warning", 0, 0, "UPS.PowerSummary.WarningCapacityLimit", NULL, "%.0f", 0, NULL },
 	{ "battery.runtime", 0, 0, "UPS.PowerSummary.RunTimeToEmpty", NULL, "%.0f", 0, NULL },
-	{ "battery.voltage",  0, 0, "UPS.BatterySystem.Battery.Voltage", NULL, "%s", 0, divide_by_10_conversion },
-	/* FIXME: the divide_by_10_conversion is used because of a bug in
-		the Tripplite Report Descriptor (the unit exponent should be -1,
-		but is not set.) This should be handled more flexibly, because
-      some devices have the bug and others don't */
-	{ "battery.voltage.nominal", 0, 0, "UPS.BatterySystem.Battery.ConfigVoltage", NULL, "%.1f", 0, NULL },
-	{ "battery.type", 0, 0, "UPS.PowerSummary.iDeviceChemistry", NULL, "%s", 0, tripplite_chemistry },
+	{ "battery.voltage.nominal", 0, 0, "UPS.BatterySystem.Battery.ConfigVoltage", NULL, "%.1f", HU_FLAG_STATIC, NULL },
+	{ "battery.voltage",  0, 0, "UPS.BatterySystem.Battery.Voltage", NULL, "%s", 0, tripplite_battvolt },
+	{ "battery.type", 0, 0, "UPS.PowerSummary.iDeviceChemistry", NULL, "%s", HU_FLAG_STATIC, tripplite_chemistry },
 	{ "battery.temperature", 0, 0, "UPS.BatterySystem.Temperature", NULL, "%s", 0, kelvin_celsius_conversion },
-	
+
 	/* UPS page */
-	{ "ups.delay.shutdown", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, "%.0f", 0, NULL},
-	{ "ups.delay.start", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, "%.0f", 0, NULL},
-	{ "ups.delay.reboot", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeReboot", NULL, "%.0f", 0, NULL },
+	{ "ups.delay.start", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, DEFAULT_ONDELAY, HU_FLAG_ABSENT, NULL},
+	{ "ups.delay.shutdown", ST_FLAG_RW | ST_FLAG_STRING, 10, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, DEFAULT_OFFDELAY, HU_FLAG_ABSENT, NULL},
+	{ "ups.timer.start", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, "%.0f", 0, NULL},
+	{ "ups.timer.shutdown", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, "%.0f", 0, NULL},
+	{ "ups.timer.reboot", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeReboot", NULL, "%.0f", 0, NULL },
 	{ "ups.test.result", 0, 0, "UPS.BatterySystem.Test", NULL, "%s", 0, test_read_info },
 	{ "ups.beeper.status", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "%s", 0, beeper_info },
-	{ "ups.power.nominal", 0, 0, "UPS.Flow.ConfigApparentPower", NULL, "%.0f", 0, NULL },
+	{ "ups.power.nominal", 0, 0, "UPS.Flow.ConfigApparentPower", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "ups.power", 0, 0, "UPS.OutletSystem.Outlet.ActivePower", NULL, "%.1f", 0, NULL },
 	{ "ups.power", 0, 0, "UPS.PowerConverter.Output.ActivePower", NULL, "%.1f", 0, NULL },
 	{ "ups.load", 0, 0, "UPS.OutletSystem.Outlet.PercentLoad", NULL, "%.0f", 0, NULL },
-	
+
+	/* Number of seconds left before the watchdog reboots the UPS (0 = disabled) */
+	{ "ups.watchdog.status", 0, 0, "UPS.OutletSystem.Outlet.TLWatchdog", NULL, "%.0f", 0, NULL },
+
 	/* Special case: ups.status */
+	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.InternalFailure", NULL, NULL, 0, commfault_info }, 
+	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ShutdownImminent", NULL, NULL, 0, shutdownimm_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ACPresent", NULL, NULL, HU_FLAG_QUICK_POLL, online_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.BelowRemainingCapacityLimit", NULL, NULL, HU_FLAG_QUICK_POLL, lowbatt_info },
-	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Discharging", NULL, NULL, HU_FLAG_QUICK_POLL, discharging_info },
-	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Charging", NULL, NULL, HU_FLAG_QUICK_POLL, charging_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.FullyCharged", NULL, NULL, HU_FLAG_QUICK_POLL, fullycharged_info },
-	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.ShutdownImminent", NULL, NULL, 0, shutdownimm_info },
-	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.Overload", NULL, NULL, 0, overload_info },
-	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.VoltageOutOfRange", NULL, NULL, 0, vrange_info },
+	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Charging", NULL, NULL, HU_FLAG_QUICK_POLL, charging_info },
+	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.Discharging", NULL, NULL, HU_FLAG_QUICK_POLL, discharging_info },
+	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.FullyDischarged", NULL, NULL, HU_FLAG_QUICK_POLL, depleted_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.NeedReplacement", NULL, NULL, 0, replacebatt_info },
-	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.Boost", NULL, NULL, 0, boost_info },
-	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.Buck", NULL, NULL, 0, trim_info },
-
 	/* repeat some of the above for faulty usage codes (seen on OMNI1000LCD, untested) */
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.TLACPresent", NULL, NULL, HU_FLAG_QUICK_POLL, online_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.TLDischarging", NULL, NULL, HU_FLAG_QUICK_POLL, discharging_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.TLCharging", NULL, NULL, HU_FLAG_QUICK_POLL, charging_info },
 	{ "BOOL", 0, 0, "UPS.PowerSummary.PresentStatus.TLNeedReplacement", NULL, NULL, 0, replacebatt_info },
 
-	/* Tripp Lite specific status flags (seen on OMNI1000LCD, untested) */
+	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.VoltageOutOfRange", NULL, NULL, 0, vrange_info },
+	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.Buck", NULL, NULL, 0, trim_info },
+	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.Boost", NULL, NULL, 0, boost_info },
+	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.Overload", NULL, NULL, 0, overload_info },
+	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.Used", NULL, NULL, 0, nobattery_info },
 	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.OverTemperature", NULL, NULL, 0, overheat_info },
-	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.AwaitingPower", NULL, NULL, 0, awaitingpower_info },
 	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.InternalFailure", NULL, NULL, 0, commfault_info },
-	
+	{ "BOOL", 0, 0, "UPS.PowerConverter.PresentStatus.AwaitingPower", NULL, NULL, 0, awaitingpower_info },
+
+	/* Duplicated values
+	{ "BOOL", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.BelowRemainingCapacityLimit", NULL, NULL, HU_FLAG_QUICK_POLL, lowbatt_info },
+	{ "BOOL", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.FullyCharged", NULL, NULL, HU_FLAG_QUICK_POLL, fullycharged_info },
+	{ "BOOL", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.Charging", NULL, NULL, HU_FLAG_QUICK_POLL, charging_info },
+	{ "BOOL", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.Discharging", NULL, NULL, HU_FLAG_QUICK_POLL, discharging_info },
+	{ "BOOL", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.FullyDischarged", NULL, NULL, HU_FLAG_QUICK_POLL, depleted_info },
+	{ "BOOL", 0, 0, "UPS.BatterySystem.Battery.PresentStatus.NeedReplacement", NULL, NULL, 0, replacebatt_info },
+	 */
+
 	/* Input page */
+	{ "input.voltage.nominal", 0, 0, "UPS.PowerSummary.Input.ConfigVoltage", NULL, "%.0f", HU_FLAG_STATIC, NULL },
+	{ "input.voltage", 0, 0, "UPS.PowerSummary.Input.Voltage", NULL, "%.1f", 0, NULL },
 	{ "input.voltage", 0, 0, "UPS.PowerConverter.Input.Voltage", NULL, "%.1f", 0, NULL },
-	{ "input.voltage.nominal", 0, 0, "UPS.PowerSummary.Input.ConfigVoltage", NULL, "%.0f", 0, NULL },
 	{ "input.frequency", 0, 0, "UPS.PowerConverter.Input.Frequency", NULL, "%.1f", 0, NULL },
-	{ "input.transfer.high", ST_FLAG_RW | ST_FLAG_STRING, 5,	"UPS.PowerConverter.Output.HighVoltageTransfer", NULL, "%.1f", HU_FLAG_SEMI_STATIC, NULL },
-	{ "input.transfer.low", ST_FLAG_RW | ST_FLAG_STRING, 5,	"UPS.PowerConverter.Output.LowVoltageTransfer", NULL, "%.1f", HU_FLAG_SEMI_STATIC, NULL },
+	{ "input.transfer.low", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.LowVoltageTransfer", NULL, "%.1f", HU_FLAG_SEMI_STATIC, NULL },
+	{ "input.transfer.low.max", 0, 0, "UPS.PowerConverter.Output.TLLowVoltageTransferMax", NULL, "%.0f", HU_FLAG_STATIC, NULL },
+	{ "input.transfer.low.min", 0, 0, "UPS.PowerConverter.Output.TLLowVoltageTransferMin", NULL, "%.0f", HU_FLAG_STATIC, NULL }, 
+	{ "input.transfer.high", ST_FLAG_RW | ST_FLAG_STRING, 5, "UPS.PowerConverter.Output.HighVoltageTransfer", NULL, "%.1f", HU_FLAG_SEMI_STATIC, NULL },
+	{ "input.transfer.high.max", 0, 0, "UPS.PowerConverter.Output.TLHighVoltageTransferMax", NULL, "%.0f", HU_FLAG_STATIC, NULL },
+	{ "input.transfer.high.min", 0, 0, "UPS.PowerConverter.Output.TLHighVoltageTransferMin", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 
 	/* Output page */
-	{ "output.voltage", 0, 0, "UPS.PowerSummary.Voltage", NULL, "%.1f", 0, NULL },
+	{ "output.voltage.nominal", 0, 0, "UPS.Flow.ConfigVoltage", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "output.voltage", 0, 0, "UPS.PowerConverter.Output.Voltage", NULL, "%.1f", 0, NULL },
-	{ "output.voltage.nominal", 0, 0, "UPS.Flow.ConfigVoltage", NULL, "%.0f", 0, NULL },
+	{ "output.voltage", 0, 0, "UPS.PowerSummary.Voltage", NULL, "%.1f", 0, NULL },
 	{ "output.current", 0, 0, "UPS.PowerConverter.Output.Current", NULL, "%.2f", 0, NULL },
+	{ "output.frequency.nominal", 0, 0, "UPS.Flow.ConfigFrequency", NULL, "%.0f", HU_FLAG_STATIC, NULL },
 	{ "output.frequency", 0, 0, "UPS.PowerConverter.Output.Frequency", NULL, "%.1f", 0, NULL },
-	{ "output.frequency.nominal", 0, 0, "UPS.Flow.ConfigFrequency", NULL, "%.0f", 0, NULL },
 
 	/* instant commands. */
 	{ "test.battery.start.quick", 0, 0, "UPS.BatterySystem.Test", NULL, "1", HU_TYPE_CMD, NULL }, /* reported to work on OMNI1000 */
 	{ "test.battery.start.deep", 0, 0, "UPS.BatterySystem.Test", NULL, "2", HU_TYPE_CMD, NULL }, /* reported not to work */
 	{ "test.battery.stop", 0, 0, "UPS.BatterySystem.Test", NULL, "3", HU_TYPE_CMD, NULL }, /* reported not to work */
 	
-	{ "load.off", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, "0", HU_TYPE_CMD, NULL },
-	{ "load.on", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, "0", HU_TYPE_CMD, NULL },
+	{ "load.off.delay", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, DEFAULT_OFFDELAY, HU_TYPE_CMD, NULL },
+	{ "load.on.delay", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, DEFAULT_ONDELAY, HU_TYPE_CMD, NULL },
 
-	{ "shutdown.stayoff", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, "20", HU_TYPE_CMD, NULL },
-	{ "shutdown.return", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeStartup", NULL, "30", HU_TYPE_CMD, NULL },
-	{ "shutdown.reboot", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeReboot", NULL, "10", HU_TYPE_CMD, NULL },
 	{ "shutdown.stop", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeShutdown", NULL, "-1", HU_TYPE_CMD, NULL },
-	
+	{ "shutdown.reboot", 0, 0, "UPS.OutletSystem.Outlet.DelayBeforeReboot", NULL, "10", HU_TYPE_CMD, NULL },
+
+	/* WARNING: if this timer expires, the UPS will reboot! Defaults to 60 seconds */
+	{ "reset.watchdog", 0, 0, "UPS.OutletSystem.Outlet.TLWatchdog", NULL, "60", HU_TYPE_CMD, NULL },
+
 	{ "beeper.on", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "2", HU_TYPE_CMD, NULL },
 	{ "beeper.off", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "3", HU_TYPE_CMD, NULL },
 	{ "beeper.disable", 0, 0, "UPS.PowerSummary.AudibleAlarmControl", NULL, "1", HU_TYPE_CMD, NULL },
@@ -228,17 +263,22 @@ static int tripplite_claim(HIDDevice_t *hd) {
 	if (hd->VendorID != TRIPPLITE_VENDORID) {
 		return 0;
 	}
-	switch (hd->ProductID)
-	{
+
 	/* accept any known UPS - add devices here as needed.
 	   Remember: also update scripts/udev/nutusb-ups.rules.in
 	   and scripts/hotplug/libhid.usermap */
+	switch (hd->ProductID)
+	{
 	case 0x1003:  /* e.g. AVR550U */
 	case 0x2005:  /* e.g. OMNI1000LCD */
 	case 0x2007:  /* e.g. OMNI900LCD */
+		battery_scale = 0.1;
+		return 1;
+
 	case 0x3012:  /* e.g. smart2200RMXL2U */
 	case 0x4002:  /* e.g. SmartOnline SU6000RT4U? */
 	case 0x4003:  /* e.g. SmartOnline SU1500RTXL2ua */
+		battery_scale = 1.0;
 		return 1;
 
 	/* reject known non-HID devices */
