@@ -200,7 +200,7 @@ static struct {
 #define BOOT_DELAY                   "UBD" /* poll/set */
 #define BAUD_RATE                    "UBR" /* poll/set */
 #define IDENTIFICATION               "UID" /* poll/set */
-#define VERSION                      "VER" /* poll */
+#define VERSION_CMD                  "VER" /* poll */
 #define VOLTAGE_SENSITIVITY          "VSN" /* poll/set */
 #define WATCHDOG                     "WDG" /* poll/set */
 
@@ -225,7 +225,7 @@ static int do_command(char type, const char *command,
 	buffer[4] = '\0';
 
 	if (!strcmp(buffer, "~00D")) {
-		if (ser_get_buf_len(upsfd, buffer, 4, 3, 0) <= 0)
+		if (ser_get_buf_len(upsfd, buffer, 3, 3, 0) <= 0)
 			return -1;
 		buffer[3] = '\0';
 		count = atoi(buffer);
@@ -328,7 +328,7 @@ static void set_transfer_voltage_low(int val) {
 	if (!ptr)
 		return;
 	high = atoi(ptr);
-	sprintf(response, "%d;%d", val, high);
+	snprintf(response, sizeof(response), "%d;%d", val, high);
 	do_command(SET, TRANSFER_VOLTAGE, response, NULL);
 }
 
@@ -360,7 +360,7 @@ static void set_transfer_voltage_high(int val) {
 	ptr = field(response, 1);
 	if (!ptr || val == atoi(ptr))
 		return;
-	sprintf(response, "%d;%d", low, val);
+	snprintf(response, sizeof(response), "%d;%d", low, val);
 	do_command(SET, TRANSFER_VOLTAGE, response, NULL);
 }
 
@@ -387,7 +387,7 @@ static void set_sensitivity(const char *val) {
 
 	for (i = 0; i < sizeof(sensitivity) / sizeof(sensitivity[0]); i++) {
 		if (!strcasecmp(val, sensitivity[i].name)) {
-			sprintf(parm, "%d", i);
+			snprintf(parm, sizeof(parm), "%d", i);
 			do_command(SET, VOLTAGE_SENSITIVITY, parm, NULL);
 			break;
 		}
@@ -408,7 +408,7 @@ static void auto_reboot(int enable) {
 		return;
 	ptr = field(response, 0);
 	if (!ptr || atoi(ptr) != mode) {
-		sprintf(parm, "%d", mode);
+		snprintf(parm, sizeof(parm), "%d", mode);
 		do_command(SET, AUTO_REBOOT, parm, NULL);
 	}
 }
@@ -420,14 +420,14 @@ static int instcmd(const char *cmdname, const char *extra)
 
 	if (!strcasecmp(cmdname, "load.off")) {
 		for (i = 0; i < ups.outlet_banks; i++) {
-			sprintf(parm, "%d;1", i + 1);
+			snprintf(parm, sizeof(parm), "%d;1", i + 1);
 			do_command(SET, RELAY_OFF, parm, NULL);
 		}
 		return STAT_INSTCMD_HANDLED;
 	}
 	if (!strcasecmp(cmdname, "load.on")) {
 		for (i = 0; i < ups.outlet_banks; i++) {
-			sprintf(parm, "%d;1", i + 1);
+			snprintf(parm, sizeof(parm), "%d;1", i + 1);
 			do_command(SET, RELAY_ON, parm, NULL);
 		}
 		return STAT_INSTCMD_HANDLED;
@@ -531,7 +531,7 @@ void upsdrv_initinfo(void)
 	char *ptr;
 
 	if (!init_comm())
-		fatalx("Unable to detect Tripp Lite SmartOnline UPS on port %s\n",
+		fatalx(EXIT_FAILURE, "Unable to detect Tripp Lite SmartOnline UPS on port %s\n",
 		        device_path);
 	min_low_transfer = max_low_transfer = 0;
 	min_high_transfer = max_high_transfer = 0;
@@ -542,7 +542,7 @@ void upsdrv_initinfo(void)
 		dstate_setinfo("ups.mfr", "%s", response);
 	if (do_command(POLL, MODEL, "", response) > 0)
 		dstate_setinfo("ups.model", "%s", response);
-	if (do_command(POLL, VERSION, "", response) > 0)
+	if (do_command(POLL, VERSION_CMD, "", response) > 0)
 		dstate_setinfo("ups.firmware", "%s", response);
 	if (do_command(POLL, RATINGS, "", response) > 0) {
 		ptr = field(response, 0);
@@ -785,9 +785,9 @@ void upsdrv_shutdown(void)
 	auto_reboot(1);
 	/* in case the power is on, tell it to automatically reboot.  if
 	   it is off, this has no effect. */
-	sprintf(parm, "%d", 1); /* delay before reboot, in minutes */
+	snprintf(parm, sizeof(parm), "%d", 1); /* delay before reboot, in minutes */
 	do_command(SET, SHUTDOWN_RESTART, parm, NULL);
-	sprintf(parm, "%d", 5); /* delay before shutdown, in seconds */
+	snprintf(parm, sizeof(parm), "%d", 5); /* delay before shutdown, in seconds */
 	do_command(SET, SHUTDOWN_ACTION, parm, NULL);
 }
 
