@@ -27,9 +27,9 @@
 
 #include "netget.h"
 
-static void get_numlogins(ctype *client, const char *upsname)
+static void get_numlogins(ctype_t *client, const char *upsname)
 {
-	const	upstype	*ups;
+	const	upstype_t	*ups;
 
 	ups = get_ups_ptr(upsname);
 
@@ -44,9 +44,9 @@ static void get_numlogins(ctype *client, const char *upsname)
 	sendback(client, "NUMLOGINS %s %d\n", upsname, ups->numlogins);
 }
 
-static void get_upsdesc(ctype *client, const char *upsname)
+static void get_upsdesc(ctype_t *client, const char *upsname)
 {
-	const	upstype	*ups;
+	const	upstype_t	*ups;
 	char	esc[SMALLBUF];
 
 	ups = get_ups_ptr(upsname);
@@ -69,9 +69,9 @@ static void get_upsdesc(ctype *client, const char *upsname)
 	}
 }
 
-static void get_desc(ctype *client, const char *upsname, const char *var)
+static void get_desc(ctype_t *client, const char *upsname, const char *var)
 {
-	const	upstype	*ups;
+	const	upstype_t	*ups;
 	const	char	*desc;
 
 	ups = get_ups_ptr(upsname);
@@ -92,9 +92,9 @@ static void get_desc(ctype *client, const char *upsname, const char *var)
 		sendback(client, "DESC %s %s \"Description unavailable\"\n", upsname, var);
 }
 
-static void get_cmddesc(ctype *client, const char *upsname, const char *cmd)
+static void get_cmddesc(ctype_t *client, const char *upsname, const char *cmd)
 {
-	const	upstype	*ups;
+	const	upstype_t	*ups;
 	const	char	*desc;
 
 	ups = get_ups_ptr(upsname);
@@ -116,10 +116,10 @@ static void get_cmddesc(ctype *client, const char *upsname, const char *cmd)
 			upsname, cmd);
 }
 
-static void get_type(ctype *client, const char *upsname, const char *var)
+static void get_type(ctype_t *client, const char *upsname, const char *var)
 {
 	char	buf[SMALLBUF];
-	const	upstype	*ups;
+	const	upstype_t	*ups;
 	const	struct	st_tree_t	*node;
 
 	ups = get_ups_ptr(upsname);
@@ -159,7 +159,7 @@ static void get_type(ctype *client, const char *upsname, const char *var)
 	sendback(client, "TYPE %s %s UNKNOWN\n", upsname, var);
 }		
 
-static void get_var_server(ctype *client, const char *upsname, const char *var)
+static void get_var_server(ctype_t *client, const char *upsname, const char *var)
 {
 	if (!strcasecmp(var, "server.info")) {
 		sendback(client, "VAR %s server.info "
@@ -178,9 +178,9 @@ static void get_var_server(ctype *client, const char *upsname, const char *var)
 	send_err(client, NUT_ERR_VAR_NOT_SUPPORTED);
 }
 
-static void get_var(ctype *client, const char *upsname, const char *var)
+static void get_var(ctype_t *client, const char *upsname, const char *var)
 {
-	const	upstype	*ups;
+	const	upstype_t	*ups;
 	const	char	*val;
 
 	/* ignore upsname for server.* variables */
@@ -213,7 +213,7 @@ static void get_var(ctype *client, const char *upsname, const char *var)
 		sendback(client, "VAR %s %s \"%s\"\n", upsname, var, val);
 }
 
-void net_get(ctype *client, int numarg, const char **arg)
+void net_get(ctype_t *client, int numarg, const char **arg)
 {
 	if (numarg < 2) {
 		send_err(client, NUT_ERR_INVALID_ARGUMENT);
